REM Created by Nicholas "Tiger" Gautier
REM Contact: SibTiger_And_Life@Hotmail.com
REM Website: http://tigersfiles.webs.com/
REM Details: This program allows the user to compile a developmental build of Doom Builder 2 software directly from the subversion local working copy, this program is essentially a complexed makefile written in Batch.
REM Prerequisites: 
REM     Bootless Star 1.5 or later
REM         http://forum.drdteam.org/viewtopic.php?p=53914#p53914
REM     Microsoft Visual Studio 2008 or later
REM         http://www.visualstudio.com/en-us/visual-studio-homepage-vs.aspx
REM     Subversion Commandline tools [OPTIONAL]
REM         http://www.tortoisesvn.net/
REM     Microsoft HTML Workshop
REM         http://www.microsoft.com/en-us/download/details.aspx?id=21138
REM     Inno Setup [OPTIONAL]
REM         http://www.jrsoftware.org/isinfo.php
REM     7Zip 9.20 preferred [OPTIONAL]
REM         http://7-zip.org/
REM     Doom Builder 2 resources [SVN Local Working Copy]
REM         http://svn.code.sf.net/p/doombuilder/code/
REM ==========================================


REM Start at Main
GOTO :Main




REM =====================================================================
REM Terms of Agreement (AKA Legal Crap)
REM ----------------------------
REM Self explanatory....
REM =====================================================================


REM I have removed the code and the terms of agreement wording from this section.  I am sure that it is not accurate and I am not a lawyer, but instead I am just a creator.  I have created this program for several reasons:
REM    A} To make some tedious tasks - less tedious and expeditious
REM    B} To help others get the job[s] finished
REM    C} Allow everyone to use this program and even further expand it for their reasons or purposes
REM
REM In exchange all that I ask for those wanting to create a script or module [I can't force anyone to follow these]:
REM    A} When you create a module or script that is designed for this program, do NOT mislead the user.  Thus, do not cause intentional harm to the user our their system.
REM    B} Modules and scripts source code should always be available.  With batch, that shouldn't be much of a problem as the scripts are already readable.
REM
REM If someone wishes to take this entire program and up[date|grade] it or change it for whatever reasons, feel free to-do so.  Do not, however, charge users (currency wise) for using and retrieving the program.  I [the creator of this program] would like _some_ credit for the main work, the meaning of _some_ can be a simple comment within the program.




REM # =============================================================================================
REM # Documentation: The spine of the program; the main is the general flow of the program but not the center of the program.  When this module starts, main will always be the function to execute.  Not only is this easier on the eyes, but this will help the programmer or user to not 'get lost' when reading the code.
REM # =============================================================================================
:Main
REM Clear the terminal buffer
CALL :ClearBuffer

REM Setup the identifiers as needed for this program
CALL :Initialization_Driver Load

REM Check and manage any protocols necessary within the prerequisite manager.
CALL :Prerequisite_Driver || SET FatalExit=1

REM Open the main menu and drive the program from there.
IF %FatalExit% EQU 0 CALL :MainMenu

REM Set the termination process.
CALL :Terminate_MainExit

REM terminate the script with the exitcode obtained from the Termination process.
EXIT /B %scriptExitCode%




REM =====================================================================
REM Prerequisites Check
REM ----------------------------
REM Check if wither or not the user meets all of the necessary prerequisites.  If the user is missing the necessary prerequisites - then the users session will ultimately terminate.  Other prerequisites, however, such as local directories can be created and setup for the program's use exclusively.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: This function will run through the prerequisite driver and make sure that this program's dependencies are meet.
REM # =============================================================================================
:Prerequisite_Driver
REM Can we find the main project?
CALL :Prerequisites_DoomBuilder2Project
REM Does the Local Directories exists?
CALL :Prerequisites_LocalDirectory
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Detect if the program can find the project's main solution files.
REM # =============================================================================================
:Prerequisites_DoomBuilder2Project
CALL :DetectionProject "%UserConfig.DirProjectWorkingCopy%"
IF %ERRORLEVEL% EQU 1 (
    SET Detect_ProjectCore=True
) ELSE (
    SET Detect_ProjectCore=False
)
GOTO :EOF



REM # =============================================================================================
REM # Documentation: If the local directories where not created, then display a warning message.
REM # =============================================================================================
:Prerequisites_LocalDirectory_CheckErr
ECHO !ERR_CRIT!: Could not create dependable directories!
ECHO              %LocalDirectory.MainRoot%
ECHO              %LocalDirectory.Temp%
ECHO              %LocalDirectory.UserConfig%
ECHO              %LocalDirectory.Archives%
ECHO              %LocalDirectory.Setup%
ECHO              %LocalDirectory.Builds%
ECHO              %LocalDirectory.Logs%
PAUSE
EXIT /B 1



REM # =============================================================================================
REM # Documentation: This function will check to make sure if the local directories exists within the filesystem, if - however they do not exist, then the directories will be created.  This local directories are crucial for this program as they are used for storing temporary data and output data.
REM # =============================================================================================
:Prerequisites_LocalDirectory
IF NOT EXIST "%LocalDirectory.MainRoot%" (
    MKDIR "%LocalDirectory.MainRoot%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
IF NOT EXIST "%LocalDirectory.Temp%" (
    MKDIR "%LocalDirectory.Temp%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
IF NOT EXIST "%LocalDirectory.UserConfig%" (
    MKDIR "%LocalDirectory.UserConfig%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
IF NOT EXIST "%LocalDirectory.Archives%" (
    MKDIR "%LocalDirectory.Archives%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
IF NOT EXIST "%LocalDirectory.Setup%" (
    MKDIR "%LocalDirectory.Setup%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
IF NOT EXIST "%LocalDirectory.Builds%" (
    MKDIR "%LocalDirectory.Builds%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
IF NOT EXIST "%LocalDirectory.Logs%" (
    MKDIR "%LocalDirectory.Logs%" || GOTO :Prerequisites_LocalDirectory_CheckErr
)
EXIT /B 0




REM =====================================================================
REM Global Functions
REM ----------------------------
REM These functions listed below is used to reduce redundancy within the code.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: This function simply clears the terminal buffer and also the StandardIn [STDIN] identifier.
REM # =============================================================================================
:ClearBuffer
CLS
REM Clear the STDIN
SET STDIN=
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function captures the user's input from the keyboard.
REM # =============================================================================================
:UserInput
ECHO.
ECHO %SeparatorSmall%
SET /P STDIN=^>^>^>^> 
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function determines wither to render the Dashboard Viewer tool or only the classical header.  This is determined by the users settings based from the core, not this module.
REM # =============================================================================================
:DashboardOrClassicalDisplay
IF %DashboardViewerTool% EQU True (
    CALL :Dashboard
) ELSE (
    ECHO %ProjectName% Module Version: %ProjectVersion%
)
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} TargetPath]
REM # Documentation: This determines wither or not the program can locate the projects directory.  The detection used within this function requires a parameter that specifies a path target, the target will be tested by finding 'builder.sln' file.
REM # =============================================================================================
:DetectionProject
IF EXIST "%~1\Builder.sln" (
    REM Passed
    EXIT /B 1
) ELSE (
    REM Failed
    EXIT /B 0
)



REM # =============================================================================================
REM # Parameters: [{Bool} ExitCode]
REM # Documentation: This will convert an boolean value to a string, the value will be stored in a working variable, ProcessVarA.
REM # =============================================================================================
:ExitCodeIntToStringCommon
IF %~1 EQU 0 SET ProcessVarA=False& GOTO :EOF
IF %~1 EQU 1 SET ProcessVarA=True& GOTO :EOF
REM Unknown
SET ProcessVarA=UNKNOWN_INT& GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will be called when the user's input does not meet the criteria that the program was wanting.  This is typically used when using the menu selections.
REM # =============================================================================================
:BadInput
ECHO.&ECHO.
ECHO ^<!^>       Invalid Input       ^<!^>
ECHO %SeparatorLong%
ECHO.
ECHO !ERR: Incorrect feedback from the user.
ECHO User Feedback: "%STDIN%"
ECHO.
PAUSE
CALL :ClearBuffer
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} Target]
REM # Documentation: This function will call the Windows Explorer and open the destination that is given when calling this function with a parameter.
REM # =============================================================================================
:Call_WindowsExplorer
EXPLORER "%~1"
GOTO :EOF




REM =====================================================================
REM Dashboard
REM ----------------------------
REM The dashboard here is to only highlight important and display what the user needs to know for the program to work correctly.
REM The idea of this dashboard, is to NOT be so verbose or full of details, but only display quick and brief details.
REM This feature can be rather intensive as this does detections, which requires reads and writes within the memory field.  Disabling this feature can slightly improve the performance.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: The main header for the Dashboard Viewer tool.
REM # =============================================================================================
:Dashboard
CALL :Dashboard_HeaderBorder
ECHO                  Dashboard Viewer
ECHO Ver: %ProjectVersion%                           %ReleaseDate%
CALL :Dashboard_ProjectDetection
ECHO %ProjectName% Directory - Status: %ProcessVarA%
ECHO Preset: %UserConfigurationLoaded%
CALL :Dashboard_CallDetection
CALL :Dashboard_FooterBorder
CALL :Dashboard_ExtraLineBreaks
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function tries to find errors that could prevent the program from working efficiently.  If errors were found, this will add warning notifications within the dashboard.
REM # =============================================================================================
:Dashboard_CallDetection
CALL :DashboardDetection
IF %ERRORLEVEL% EQU 1 (
    REM Some resources didn't pass through the detection
    ECHO.
    ECHO Warning^(s^): %ProcessVarB%
    ECHO %SeparatorSmall%
    ECHO %ProcessVarA%
)
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will display wither or not the main project can be located; if the detection fails, this will warn the user.
REM # =============================================================================================
:Dashboard_ProjectDetection
CALL :DetectionProject "%UserConfig.DirProjectWorkingCopy%"
IF %ERRORLEVEL% EQU 1 (
    SET ProcessVarA=Found
) ELSE (
    SET ProcessVarA=Not Found
)
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This adds additional lines to help prevent the Dashboard Viewer tool from being crowded with the main content of the program.
REM # =============================================================================================
:Dashboard_ExtraLineBreaks
ECHO.&ECHO.&ECHO.
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function adds the header to the main Dashboard Viewer tool.
REM # =============================================================================================
:Dashboard_HeaderBorder
ECHO %SeparatorLong%
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function adds the footer to the main Dashboard Viewer tool.
REM # =============================================================================================
:Dashboard_FooterBorder
ECHO %SeparatorLong%
ECHO %SeparatorLong%
GOTO :EOF




REM =====================================================================
REM Dashboard - Detection
REM ----------------------------
REM This is merely an extended version of the dashboard feature, though this includes further checks.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: This function executes the entire detection procedure for the Dashboard Viewer Tool.
REM # =============================================================================================
:DashboardDetection
CALL :DashboardDetection_CheckResources
IF %ProcessVarB% NEQ 0 EXIT /B 1
EXIT /B 0



REM # =============================================================================================
REM # Documentation: This function scans for warnings that could cause problems when attempting to compile the project.
REM # =============================================================================================
:DashboardDetection_CheckResources
REM Clear the variables
SET ProcessVarA=
SET ProcessVarB=0

CALL :DashboardDetection_dotNETFoundation3-5
IF %ERRORLEVEL% EQU 1 (
    SET "ProcessVarA=%ProcessVarA%.NET Framework 3.5 - Not Found!&ECHO."
    SET /A ProcessVarB=%ProcessVarB%+1
)
CALL :DashboardDetection_VisualStudio
IF %ERRORLEVEL% EQU 1 (
    SET "ProcessVarA=%ProcessVarA%Visual Studio - Not Found!&ECHO."
    SET /A ProcessVarB=%ProcessVarB%+1
)
CALL :DashboardDetection_WorkingCopy
IF %ERRORLEVEL% EQU 1 (
    SET "ProcessVarA=%ProcessVarA%Doom Builder 2 Dir - Not Found!&ECHO."
    SET /A ProcessVarB=%ProcessVarB%+1
)
CALL :DashboardDetection_SVN
IF %ERRORLEVEL% EQU 1 (
    SET "ProcessVarA=%ProcessVarA%Subversion - Not Found!&ECHO."
    SET /A ProcessVarB=%ProcessVarB%+1
)
CALL :DashboardDetection_7Zip
IF %ERRORLEVEL% EQU 1 (
    SET "ProcessVarA=%ProcessVarA%7Zip - Not Found!&ECHO."
    SET /A ProcessVarB=%ProcessVarB%+1
)
CALL :DashboardDetection_InnoSetup
IF %ERRORLEVEL% EQU 1 (
    SET "ProcessVarA=%ProcessVarA%Inno Setup - Not Found!&ECHO."
    SET /A ProcessVarB=%ProcessVarB%+1
)
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Were we able to detect the project's Working Copy files?
REM # =============================================================================================
:DashboardDetection_WorkingCopy
REM Doom Builder 2 Local Working Copy
CALL :DetectionProject "%UserConfig.DirProjectWorkingCopy%"
IF %ERRORLEVEL% EQU 0 EXIT /B 1
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Was the Subversion Commandline tools detected from the core?
REM # =============================================================================================
:DashboardDetection_SVN
IF %UserConfig.SVNMasterControl% EQU True (
    IF %Detect_SVN% EQU False EXIT /B 1
)
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Was the 7Zip software detected from the core?
REM # =============================================================================================
:DashboardDetection_7Zip
IF %UserConfig.7ZipMasterControl% EQU True (
    IF %Detect_7Zip% EQU False EXIT /B 1
)
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Was the Inno Setup software detected from the core?
REM # =============================================================================================
:DashboardDetection_InnoSetup
IF %UserConfig.InnoMasterControl% EQU True (
    IF %Detect_InnoSetup% EQU False EXIT /B 1
)
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Was Microsoft .NET Framework 3.5 detected from the core?
REM # =============================================================================================
:DashboardDetection_dotNETFoundation3-5
IF %Detect_dotNET3.5% EQU False (
    EXIT /B 1
) ELSE (
    EXIT /B 0
)



REM # =============================================================================================
REM # Documentation: Was a product of Microsoft Visual Studio selected?
REM # =============================================================================================
:DashboardDetection_VisualStudio
IF NOT DEFINED VisualStudio (
    EXIT /B 1
) ELSE (
    EXIT /B 0
)




REM =====================================================================
REM Initialization Driver
REM ----------------------------
REM This driver will manage how to either initialize identifiers or if the configuration needs to be saved.
REM =====================================================================


REM # =============================================================================================
REM # Parameters: [{String} Mode] [{String} Options] [{String} TargetPath]
REM # Documentation: This function, depending how it is called - which is set by the first parameter [MODE], will help set up the environment for the program, create or update a user configuration, or update the project resource targets [plugins, directories, etc].
REM # =============================================================================================
:Initialization_Driver
REM This driver requires parameters:
REM     [Mode] (OPTIONS) (PATH)
REM When starting the program, this function will be executed.
IF %1 EQU Load (
    REM Parameters: [Mode = Load]
    REM Load the environment field when starting up the program or when this block is executed.
    CALL :Initialization_IdentifiersStatic
    CALL :Initialization_IdentifiersUserSettings 0
    CALL :Initialization_LocalDirectories
    CALL :UserPreset_Driver 0
    CALL :Initialization_ProjectDirectories
    CALL :Initialization_UpdateLogging
    GOTO :EOF
)
IF %1 EQU SaveUserConfig (
    REM Parameters: [Mode = SaveUserConfig] [File to Output]
    REM Write the current user configuration that is in current active memory and write it to a file.
    CALL :Initialization_IdentifiersUserSettings 1 "%~2"
    GOTO :EOF
)
IF %1 EQU UpdateProjectTargets (
    CALL :Initialization_ProjectDirectories
    GOTO :EOF
)



REM =====================================================================
REM Module Initialization Functions
REM ----------------------------
REM These functions are designed to help setup the environment
REM =====================================================================


REM # =============================================================================================
REM # Documentation: This function sets the static environment for the module, and other variables that is self managed 1ithin the program.
REM # =============================================================================================
:Initialization_IdentifiersStatic
REM Static Variables
SET ProjectName=Doom Builder 2
SET ProjectNameShort=DB2
SET ProjectVersion=1.5.4
SET ReleaseDate=26.March.2024
SET ProjectNameCompact=DoomBuilder2
REM Used as the filename and directory when compiling the project.  This is built in the compiling protocol, just let it as 'UNKNOWN' here and change it later.
SET FileName=UNKNOWN
SET FileName_Archive=UNKNOWN
SET FileName_Setup=UNKNOWN
REM Error Signal for Operations; if the value is not null, there was a general issue with a function or functions in sequence.
SET ErrorSignal=0
REM Exit Code captures ErrorLevel - but retains the xid.
SET ExitCode=0
REM Fatal Exit retains the exit code from the lower level [spine of the program]; if there is an error from the lower level, then this variable will be updated.
SET FatalExit=0
REM What internal or external CMD was called, this is going to be used primarily for logs and error messages.  Make this easy for the user to understand what program was called, do not make it complicated.
SET TaskCaller_NiceProgramName=NULL
REM Compiled version of 'TaskCaller' into one string.
SET TaskCaller_CallLong=NULL
REM Static SVN revision number holders
SET SVNRevisionOld=-1
SET SVNRevisionNew=-1
SET SVNRevisionRange=0
REM A set hard limit for retrieving the SVN change log history
SET SVNRevisionRangeHardLimit=50
REM Can we update the cached revision that is stored in a small ASCII file?
SET SVNRevisionUpdateCachedRevisionID=False
REM This houses the parameters for 7Zip, this gets filled when we need to invoke it.
SET SevenZipCompactParameters=-
REM Can we find the main project?
SET Detect_ProjectCore=False
REM Update the internal project version ID?
SET ProjectModify.UpdateInternalProjectVersion=False
REM Parameters to be used when using MSBuild from Microsoft Visual Studio packages.
SET "MSVSArg=/t:Rebuild /p:Configuration=Release;Platform=x86 /v:quiet"
REM Parameters to be used when using VCUpgrade from Microsoft Visual Studio packages.
SET "MSVSArgVPO=/p:Configuration=Release;Platform=Win32 /v:quiet"
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET DriversNiceTask=UNKNOWN
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will set the Local Directory targets.  These local directories are necessary for the program to work correctly.
REM # =============================================================================================
:Initialization_LocalDirectories
REM Note: DirLocal is merely a simple dedicated directory for output data and saved data.
SET "LocalDirectory.MainRoot=%DirProjects%\%ProjectNameCompact%"
REM Temporary data
SET "LocalDirectory.Temp=%LocalDirectory.MainRoot%\Temp"
REM Preset Configurations
SET "LocalDirectory.UserConfig=%LocalDirectory.MainRoot%\Config"
REM Archive builds [compiled]
SET "LocalDirectory.Archives=%LocalDirectory.MainRoot%\Archives"
REM Inno Setup Builds
SET "LocalDirectory.Setup=%LocalDirectory.MainRoot%\Setup"
REM Regular data files
SET "LocalDirectory.Builds=%LocalDirectory.MainRoot%\Builds"
REM Log Files
SET "LocalDirectory.Logs=%LocalDirectory.MainRoot%\Logs"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function holds the project's resource targets: important directories, executable binaries, solution projects, assembly files, etc.
REM # =============================================================================================
:Initialization_ProjectDirectories
REM Doom Builder 2 Directories
REM NOTE: The root directory is defined within the user configuration.
REM   VersionFromSVN
SET "ProjectDirectory.VersionFromSVN=%UserConfig.DirProjectWorkingCopy%\VersionFromSVN.exe"
REM   Directories
SET "ProjectDirectory.DirCompiledTarget=%UserConfig.DirProjectWorkingCopy%\Build"
SET "ProjectDirectory.DirCompileSetupTarget=%ProjectDirectory.DirCompiledTarget%\Setup"
SET "ProjectDirectory.DirPluginsTarget=%UserConfig.DirProjectWorkingCopy%\Source\Plugins"
SET "ProjectDirectory.DirHelpTarget=%UserConfig.DirProjectWorkingCopy%\Help"
SET "ProjectDirectory.DirSetupTarget=%UserConfig.DirProjectWorkingCopy%\Setup"
SET "ProjectDirectory.DirReleaseTarget=%UserConfig.DirProjectWorkingCopy%\Release"
REM ------------
REM   Core
REM ------------
SET "ProjectDirectory.PathCoreEngine=%UserConfig.DirProjectWorkingCopy%\Source\Core\Builder.csproj"
SET "ProjectDirectory.PathCoreAssembly=%UserConfig.DirProjectWorkingCopy%\Source\Core\Properties\AssemblyInfo.cs"
REM ------------
REM   Plugins
REM ------------
REM    Builder Modes
SET "ProjectDirectory.PathPluginBuilderModes=%ProjectDirectory.DirPluginsTarget%\BuilderModes\BuilderModes.csproj"
SET "ProjectDirectory.PathPluginBuilderModesAssembly=%ProjectDirectory.DirPluginsTarget%\BuilderModes\Properties\AssemblyInfo.cs"
REM ----
REM    Comments Panel
SET "ProjectDirectory.PathPluginCommentsPanel=%ProjectDirectory.DirPluginsTarget%\CommentsPanel\CommentsPanel.csproj"
SET "ProjectDirectory.PathPluginCommentsPanelAssembly=%ProjectDirectory.DirPluginsTarget%\CommentsPanel\Properties\AssemblyInfo.cs"
REM ----
REM    Copy and Paste Sector Properties
SET "ProjectDirectory.PathPluginCopyPaste=%ProjectDirectory.DirPluginsTarget%\CopyPasteSectorProps\CopyPasteSectorProperties.csproj"
SET "ProjectDirectory.PathPluginCopyPasteAssembly=%ProjectDirectory.DirPluginsTarget%\CopyPasteSectorProps\Properties\AssemblyInfo.cs"
REM ----
REM    Image Drawing Example
SET "ProjectDirectory.PathPluginImageDraw=%ProjectDirectory.DirPluginsTarget%\ImageDrawingExample\ImageDrawingExample.csproj"
SET "ProjectDirectory.PathPluginImageDrawAssembly=%ProjectDirectory.DirPluginsTarget%\ImageDrawingExample\Properties\AssemblyInfo.cs"
REM ----
REM    Statistics
SET "ProjectDirectory.PathPluginStatistics=%ProjectDirectory.DirPluginsTarget%\Statistics\Statistics.csproj"
SET "ProjectDirectory.PathPluginStatisticsAssembly=%ProjectDirectory.DirPluginsTarget%\Statistics\Properties\AssemblyInfo.cs"
REM ----
REM    TagRange
SET "ProjectDirectory.PathPluginTagRange=%ProjectDirectory.DirPluginsTarget%\TagRange\TagRange.csproj"
SET "ProjectDirectory.PathPluginTagRangeAssembly=%ProjectDirectory.DirPluginsTarget%\TagRange\Properties\AssemblyInfo.cs"
REM ----
REM    USDF
SET "ProjectDirectory.PathPluginUSDF=%ProjectDirectory.DirPluginsTarget%\USDF\USDF.csproj"
SET "ProjectDirectory.PathPluginUSDFAssembly=%ProjectDirectory.DirPluginsTarget%\USDF\Properties\AssemblyInfo.cs"
REM ----
REM    GZDoom Visual Mode
SET "ProjectDirectory.PathPluginGZDoomVirtualMode=%ProjectDirectory.DirPluginsTarget%\GZDoomEditing\GZDoomEditing.csproj"
SET "ProjectDirectory.PathPluginGZDoomVirtualModeAssembly=%ProjectDirectory.DirPluginsTarget%\GZDoomEditing\Properties\AssemblyInfo.cs"
REM ----
REM    WADAuthor Visual Mode
SET "ProjectDirectory.PathPluginWadAuthorVirtualMode=%ProjectDirectory.DirPluginsTarget%\WadAuthorMode\WadAuthorMode.csproj"
SET "ProjectDirectory.PathPluginWadAuthorVirtualModeAssembly=%ProjectDirectory.DirPluginsTarget%\WadAuthorMode\Properties\AssemblyInfo.cs"
REM ----
REM    Visplane Explorer [Dynamic Link Library]
SET "ProjectDirectory.PathPluginVisplaneExplorerLibraryDir=%ProjectDirectory.DirPluginsTarget%\vpo_dll\"
SET "ProjectDirectory.PathPluginVisplaneExplorerLibrary=%ProjectDirectory.DirPluginsTarget%\vpo_dll\vpo_dll.vcproj"
SET "ProjectDirectory.PathPluginVisplaneExplorerLibraryUpgrade=%ProjectDirectory.DirPluginsTarget%\vpo_dll\vpo_dll.vcxproj"
SET "ProjectDirectory.PathPluginVisplaneExplorerDir=%ProjectDirectory.DirPluginsTarget%\VisplaneExplorer\Resources\"
REM ----
REM    Visplane Explorer
SET "ProjectDirectory.PathPluginVisplaneExplorer=%ProjectDirectory.DirPluginsTarget%\VisplaneExplorer\VisplaneExplorer.csproj"
SET "ProjectDirectory.PathPluginVisplaneExplorerAssembly=%ProjectDirectory.DirPluginsTarget%\VisplaneExplorer\Properties\AssemblyInfo.cs"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{Int} Mode] [{String} UserConfigOutput]
REM # Documentation: This function houses the users settings and customizations; this manages both loading and writing to a file, depending on the argument when this function is called.
REM #  First Parameter:
REM #   0 = Read\Load
REM #   1 = Write to specific file
REM #  Second Parameter:
REM #   Output File
REM # =============================================================================================
:Initialization_IdentifiersUserSettings
REM --------------------------------
REM ================================
REM Subversion Settings
REM -------------------------------------
IF %1 EQU 1 (ECHO REM Subversion Settings)>> "%~2"
IF %1 EQU 1 (ECHO REM --------------------)>> "%~2"
REM ----
REM Allow the program to use Subversion commandline?
IF %1 EQU 0 (SET UserConfig.SVNMasterControl=True)
IF %1 EQU 1 (ECHO REM Allow the program to use Subversion CUI)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.SVNMasterControl=%UserConfig.SVNMasterControl%)>> "%~2"
REM ----
REM Allow the working copy to be restored to its original state?
IF %1 EQU 0 (SET UserConfig.SVNAllowWorkingCopyRevert=False)
IF %1 EQU 1 (ECHO REM Allow the program to completely revert and clean up the Working Copy?  For safety reasons, this is disabled by default, only enable this if you do NOT plan to leave uncommitted work in that entire directory tree!)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.SVNAllowWorkingCopyRevert=%UserConfig.SVNAllowWorkingCopyRevert%)>> "%~2"
REM ----
REM Allow the working copy to be updated?
IF %1 EQU 0 (SET UserConfig.SVNAllowWorkingCopyUpdate=True)
IF %1 EQU 1 (ECHO REM Allow the program to update the Working Copy.)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.SVNAllowWorkingCopyUpdate=%UserConfig.SVNAllowWorkingCopyUpdate%)>> "%~2"
REM ----
REM Allow the program to fetch for the SVN revision log history?
IF %1 EQU 0 (SET UserConfig.SVNAllowFetchRevisionLog=True)
IF %1 EQU 1 (ECHO REM This allows the program to fetch the SVN revision log.)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.SVNAllowFetchRevisionLog=%UserConfig.SVNAllowFetchRevisionLog%)>> "%~2"
REM ----
REM Allow the program to fetch for the SVN revision log history in XML format?
IF %1 EQU 0 (SET UserConfig.SVNAllowFetchRevisionLogXML=False)
IF %1 EQU 1 (ECHO REM This allows the program to fetch the SVN revision log in XML formatting.)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.SVNAllowFetchRevisionLogXML=%UserConfig.SVNAllowFetchRevisionLogXML%)>> "%~2"


IF %1 EQU 1 (ECHO.)>> "%~2"
REM 7Zip Settings
REM -------------------------------------
IF %1 EQU 1 (ECHO REM 7Zip Settings)>> "%~2"
IF %1 EQU 1 (ECHO REM --------------------)>> "%~2"
REM ----
REM Allow the program to use 7Zip?
IF %1 EQU 0 (SET UserConfig.7ZipMasterControl=False)
IF %1 EQU 1 (ECHO REM Allow the program to use 7Zip)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.7ZipMasterControl=%UserConfig.7ZipMasterControl%)>> "%~2"
REM ----
REM Allow the program to generate an archive file?
IF %1 EQU 0 (SET UserConfig.7ZipGenerateArchive=False)
IF %1 EQU 1 (ECHO REM This can allow the program to generate an archive file.)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.7ZipGenerateArchive=%UserConfig.7ZipGenerateArchive%)>> "%~2"
REM ----


IF %1 EQU 1 (ECHO.)>> "%~2"
REM Inno Setup Settings
REM -------------------------------------
IF %1 EQU 1 (ECHO REM Inno Setup Settings)>> "%~2"
IF %1 EQU 1 (ECHO REM --------------------)>> "%~2"
REM ----
REM Allow the program to use Inno Setup Generator?
IF %1 EQU 0 (SET UserConfig.InnoMasterControl=False)
IF %1 EQU 1 (ECHO REM Allow the program to use Inno Setup)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.InnoMasterControl=%UserConfig.InnoMasterControl%)>> "%~2"
REM ----
REM Allow the program to build a setup package?
IF %1 EQU 0 (SET UserConfig.InnoGenerateSetup=False)
IF %1 EQU 1 (ECHO REM Allows the program to generate a setup package.)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.InnoGenerateSetup=%UserConfig.InnoGenerateSetup%)>> "%~2"


IF %1 EQU 1 (ECHO.)>> "%~2"
REM Directory Management Settings
REM -------------------------------------
IF %1 EQU 1 (ECHO REM Directory Management Settings)>> "%~2"
IF %1 EQU 1 (ECHO REM --------------------)>> "%~2"
REM ----
REM Direct location of the project location [must be in the same path as the solution file]
IF %1 EQU 0 (SET "UserConfig.DirProjectWorkingCopy=%UserProfile%\DoomBuilder\Trunk")
IF %1 EQU 1 (ECHO REM Path to the project.  This is crucially important for the program to have the correct path, otherwise this program will not properly work.)>> "%~2"
IF %1 EQU 1 (ECHO SET "UserConfig.DirProjectWorkingCopy=%UserConfig.DirProjectWorkingCopy%")>> "%~2"
REM ----
REM Allow the program to keep the setup directory, which contains the dependencies for the compiled project.
IF %1 EQU 0 (SET UserConfig.KeepSetupDir=False)
IF %1 EQU 1 (ECHO REM Allow the program to keep the Setup Directory within the Working Copy)>> "%~2"
IF %1 EQU 1 (ECHO SET UserConfig.KeepSetupDir=%UserConfig.KeepSetupDir%)>> "%~2"
REM ----
REM Thrash the data that was transfered to the Standard Output [when only: compacting to archive or setup package]
IF %1 EQU 0 (SET UserConfig.MirrorCompiledData=True)
IF %1 EQU 1 (ECHO REM Remove the compiled data that was transported to DirLocal.)>> "%~2"
IF %1 EQU 1 (ECHO SET "UserConfig.MirrorCompiledData=%UserConfig.MirrorCompiledData%")>> "%~2"


IF %1 EQU 1 (ECHO.)>> "%~2"
REM Program Management Settings
REM -------------------------------------
IF %1 EQU 1 (ECHO REM Program Management Settings)>> "%~2"
IF %1 EQU 1 (ECHO REM --------------------)>> "%~2"
REM ----
REM Choose the default Visual Studio environment when compiling the project and its resources.
IF %1 EQU 0 CALL :VisualStudio_AutomaticDetection
IF %1 EQU 1 (ECHO REM Choose the default Visual Studio environment when compiling the project and its resources.)>> "%~2"
IF %1 EQU 1 (ECHO SET "VisualStudio=%VisualStudio%")
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Update the log file name, if logging is enabled.
REM # =============================================================================================
:Initialization_UpdateLogging
IF "%ToggleLog%" NEQ "False" SET "STDOUT=%LocalDirectory.Logs%\%ProjectNameCompact%_%core.Date%.txt"
GOTO :EOF




REM =====================================================================
REM Visual Studio Detection
REM ----------------------------
REM This entire section is detected for scanning available Visual Studio installations and choosing the appropriate version for compiling the project.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: This function will try at best to determine the best version of Visual Studio to use for the project.  As of developing this section, the best is product is: 2008 && 2010 -- Anything higher may result into issues with the .NET foundation.
REM # =============================================================================================
:VisualStudio_AutomaticDetection
REM Visual Studio Versions:
REM 2008
IF %Detect_MSVS90COMMTOOLS% EQU True (
    SET "VisualStudio=%PathMSVS2008%"
    GOTO :EOF
)

REM 2010
IF %Detect_MSVS100COMMTOOLS% EQU True (
    SET "VisualStudio=%PathMSVS2010%"
    GOTO :EOF
)

REM 2012
IF %Detect_MSVS110COMMTOOLS% EQU True (
    SET "VisualStudio=%PathMSVS2012%"
    GOTO :EOF
)

REM 2013
IF %Detect_MSVS120COMMTOOLS% EQU True (
    SET "VisualStudio=%PathMSVS2013%"
    GOTO :EOF
)
REM Could not find any suitable version
ECHO !ATTN!: Could not find any suitable Microsoft Visual Studio versions within the system!
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Find the nice name for the currently used Visual Studio version.
REM # =============================================================================================
:VisualStudio_FindNiceName
REM 2008
IF "%VisualStudio%" EQU "%PathMSVS2008%" (
    SET VisualStudioNiceName=Visual Studio 2008
    GOTO :EOF
)

REM 2010
IF "%VisualStudio%" EQU "%PathMSVS2010%" (
    SET VisualStudioNiceName=Visual Studio 2010
    GOTO :EOF
)

REM 2012
IF "%VisualStudio%" EQU "%PathMSVS2012%" (
    SET VisualStudioNiceName=Visual Studio 2012
    GOTO :EOF
)

REM 2013
IF "%VisualStudio%" EQU "%PathMSVS2013%" (
    SET VisualStudioNiceName=Visual Studio 2013
    GOTO :EOF
)
REM Could not find any suitable version
ECHO !ATTN!: Could not find the name of the current Visual Studio preferred!  As such, the name within the location is currently unknown.
SET VisualStudioNiceName=UNKNOWN
PAUSE
GOTO :EOF




REM =====================================================================
REM Main Menu Interaction Field
REM ----------------------------
REM The main menu for the module program.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: Main menu of the program.
REM # =============================================================================================
:MainMenu
CALL :DashboardOrClassicalDisplay
ECHO %ProjectName% Menu
ECHO %Separator%
ECHO.
ECHO Compile Operations
ECHO %SeparatorSmall%
ECHO [1] Compile Core only
ECHO [2] Compile Core and Default Plugins
ECHO [3] Compile Core and All Plugins
ECHO.
ECHO Other Options
ECHO %SeparatorSmall%
ECHO [S] Settings
ECHO [V] View Directories
ECHO [C] Cleanup
ECHO [U] Check for Updates
ECHO [X] Exit
CALL :UserInput
GOTO :Menu_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:Menu_UserInput
IF "%STDIN%" EQU "1" GOTO :Main_Choice_NoPlugins
IF "%STDIN%" EQU "2" GOTO :Main_Choice_DefaultSet
IF "%STDIN%" EQU "3" GOTO :Main_Choice_EverythingSet
IF /I "%STDIN%" EQU "S" GOTO :Main_Choice_Settings
IF /I "%STDIN%" EQU "V" GOTO :Main_Choice_ViewDirectories
IF /I "%STDIN%" EQU "C" GOTO :Main_Choice_Cleanup
IF /I "%STDIN%" EQU "U" GOTO :Main_Choice_Updates
IF /I "%STDIN%" EQU "X" GOTO :EOF
CALL :BadInput& GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Compile the project, but omit the plugins.
REM # =============================================================================================
:Main_Choice_NoPlugins
CALL :ClearBuffer
CALL :CompileManager_Driver 0
CALL :ClearBuffer
GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Compile the project, but only compile the default plugins.
REM # =============================================================================================
:Main_Choice_DefaultSet
CALL :ClearBuffer
CALL :CompileManager_Driver 1
CALL :ClearBuffer
GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Compile the project and every available plugin.
REM # =============================================================================================
:Main_Choice_EverythingSet
CALL :ClearBuffer
CALL :CompileManager_Driver 2
CALL :ClearBuffer
GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Access the control panel.
REM # =============================================================================================
:Main_Choice_Settings
CALL :ClearBuffer
CALL :Settings
CALL :ClearBuffer
GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Check for available module updates.
REM # =============================================================================================
:Main_Choice_Updates
CALL :ClearBuffer
CALL :ModuleUpdates
CALL :ClearBuffer
GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Cleanup the local directory and sub-directories.
REM # =============================================================================================
:Main_Choice_Cleanup
CALL :ClearBuffer
CALL :Cleanup_Menu
CALL :ClearBuffer
GOTO :MainMenu



REM # =============================================================================================
REM # Documentation: Allow the user to simply open directories in the GUI shell.
REM # =============================================================================================
:Main_Choice_ViewDirectories
CALL :ClearBuffer
CALL :ViewDirectories
CALL :ClearBuffer
GOTO :MainMenu




REM =====================================================================
REM Check for Updates
REM ----------------------------
REM To make things for the end-user, this allows the user to quickly check for available updates with ease.  Despite there's limitations with Batch, we can not really automatically fetch information and append them.  However, instead, this will open Windows Explorer's functionality to open website pages.  This functionality alone will help the user to determine if there is new updates or if the module that they currently have is still the current latest version.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: This function is a small driver for checking updates.
REM # =============================================================================================
:ModuleUpdates
CALL :DashboardOrClassicalDisplay
CALL :ModuleUpdates_InternalInformation
CALL :ModuleUpdates_OpenWeb
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This will display all of the information necessary to help the user determine if the module is up to date or is outdated.
REM # =============================================================================================
:ModuleUpdates_InternalInformation
ECHO Current Module Information:
ECHO %Separator%
ECHO Project:
ECHO    %ProjectName% [%ProjectNameShort%]
ECHO Version:
ECHO    %ProjectVersion%
ECHO Released:
ECHO    %ReleaseDate%
ECHO.&ECHO.
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will call Windows Explorer - regardless of the CallExplorer variable from the core -, and go to an official website or forum that will contain information and resources of newer updates and recent changes.
REM # =============================================================================================
:ModuleUpdates_OpenWeb
EXPLORER "http://forum.drdteam.org/viewtopic.php?p=53914#p53914"
GOTO :EOF




REM =====================================================================
REM Settings Menu
REM ----------------------------
REM This allows the user to manipulate some settings that is available in this module script.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: Main Control Panel
REM # =============================================================================================
:Settings
CALL :DashboardOrClassicalDisplay
ECHO Control Panel
ECHO %Separator%
ECHO.
ECHO [1] SVN Settings
ECHO [2] 7Zip Settings
ECHO [3] Inno Setup
ECHO [4] Visual Studio Setup
ECHO [5] Directory Management
ECHO [U] Update Saved Profile named: %UserConfigurationLoaded%
ECHO [X] Exit
CALL :UserInput
GOTO :Settings_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:Settings_UserInput
IF "%STDIN%" EQU "1" GOTO :SettingsChoice_SVN
IF "%STDIN%" EQU "2" GOTO :SettingsChoice_7Zip
IF "%STDIN%" EQU "3" GOTO :SettingsChoice_Inno
IF "%STDIN%" EQU "4" GOTO :SettingsChoice_VisualStudio
IF "%STDIN%" EQU "5" GOTO :SettingsChoice_DirectoryManagement
IF /I "%STDIN%" EQU "U" GOTO :SettingsChoice_UpdateProfile
IF /I "%STDIN%" EQU "X" GOTO :EOF
IF /I "%STDIN%" EQU "Exit" GOTO :EOF
CALL :BadInput& GOTO :Settings



REM # =============================================================================================
REM # Documentation: Subversion Settings
REM # =============================================================================================
:SettingsChoice_SVN
CALL :ClearBuffer
CALL :SettingsSVN_Menu
CALL :ClearBuffer
GOTO :Settings



REM # =============================================================================================
REM # Documentation: 7Zip Settings
REM # =============================================================================================
:SettingsChoice_7Zip
CALL :ClearBuffer
CALL :Settings7Zip_Menu
CALL :ClearBuffer
GOTO :Settings



REM # =============================================================================================
REM # Documentation: Inno Settings
REM # =============================================================================================
:SettingsChoice_Inno
CALL :ClearBuffer
CALL :SettingsInno_Menu
CALL :ClearBuffer
GOTO :Settings



REM # =============================================================================================
REM # Documentation: Visual Studio Settings
REM # =============================================================================================
:SettingsChoice_VisualStudio
CALL :ClearBuffer
CALL :SettingsVisualStudio_Menu
CALL :ClearBuffer
GOTO :Settings



REM # =============================================================================================
REM # Documentation: Directory Settings
REM # =============================================================================================
:SettingsChoice_DirectoryManagement
CALL :ClearBuffer
CALL :SettingsDirectoryManagement_Menu
CALL :ClearBuffer
GOTO :Settings



REM # =============================================================================================
REM # Documentation: User Configuration Settings
REM # =============================================================================================
:SettingsChoice_UpdateProfile
CALL :ClearBuffer
CALL :UserPreset_Driver 1
CALL :ClearBuffer
GOTO :Settings



REM ============================================================



REM # =============================================================================================
REM # Documentation: Subversion Commandline Settings Menu
REM # =============================================================================================
:SettingsSVN_Menu
CALL :DashboardOrClassicalDisplay
ECHO Subversion Control Panel
ECHO %Separator%
ECHO.
ECHO [1] SVN Master Control
ECHO     When set to 'True', this will allow the program to utilize SVN features and functionality.
ECHO     - Current Value: [%UserConfig.SVNMasterControl%]
ECHO     - Detected: [%Detect_SVN%]
ECHO.
ECHO [2] Allow Local Working Copy Upkeep
ECHO     Do not enable this if you store uncommitted or altered data within the local working copy, as _revert_ will restore the entire working copy to its original state.
ECHO     - Current Value: [%UserConfig.SVNAllowWorkingCopyRevert%]
ECHO.
ECHO [3] Allow Local Working Copy to be Updated
ECHO     Allow the local working copy to be updated to that latest revision.
ECHO     - Current Value: [%UserConfig.SVNAllowWorkingCopyUpdate%]
ECHO.
ECHO [4] Allow finding changelog history
ECHO     Allow the program to fetch a changelog ^(SVN Revision log^) history of the %ProjectName% project.
ECHO     - Current Value: [%UserConfig.SVNAllowFetchRevisionLog%]
ECHO.
ECHO [5] Allow finding changelog history in XML format
ECHO     Allow the program to fetch a changelog ^(SVN Revision log^) history of the %ProjectName% project in XML formatting.
ECHO     - Current Value: [%UserConfig.SVNAllowFetchRevisionLogXML%]
ECHO.
ECHO [X] Exit
CALL :UserInput
GOTO :SettingsSVN_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:SettingsSVN_UserInput
IF "%STDIN%" EQU "1" GOTO :SettingsSVN_ToggleMaster
IF "%STDIN%" EQU "2" GOTO :SettingsSVN_ToggleWorkingCopyRevert
IF "%STDIN%" EQU "3" GOTO :SettingsSVN_ToggleWorkingCopyUpdate
IF "%STDIN%" EQU "4" GOTO :SettingsSVN_ToggleFetchRevisionLog
IF "%STDIN%" EQU "5" GOTO :SettingsSVN_ToggleFetchRevisionLogXML
IF /I "%STDIN%" EQU "X" GOTO :EOF
IF /I "%STDIN%" EQU "Exit" GOTO :EOF
CALL :BadInput& GOTO :SettingsSVN_Menu



REM # =============================================================================================
REM # Documentation: Allow the program to use Subversion?  If this is set to false, the program is _NOT_ allowed to use Subversion commandline utilities.
REM # =============================================================================================
:SettingsSVN_ToggleMaster
IF %UserConfig.SVNMasterControl% EQU True (
    SET UserConfig.SVNMasterControl=False
) ELSE (
    SET UserConfig.SVNMasterControl=True
)
CALL :ClearBuffer
GOTO :SettingsSVN_Menu



REM # =============================================================================================
REM # Documentation: This allows the program to revert the local working copy contents.
REM # =============================================================================================
:SettingsSVN_ToggleWorkingCopyRevert
IF %UserConfig.SVNAllowWorkingCopyRevert% EQU True (
    SET UserConfig.SVNAllowWorkingCopyRevert=False
) ELSE (
    SET UserConfig.SVNAllowWorkingCopyRevert=True
)
CALL :ClearBuffer
GOTO :SettingsSVN_Menu



REM # =============================================================================================
REM # Documentation: This allows the program to update the local working copy contents.
REM # =============================================================================================
:SettingsSVN_ToggleWorkingCopyUpdate
IF %UserConfig.SVNAllowWorkingCopyUpdate% EQU True (
    SET UserConfig.SVNAllowWorkingCopyUpdate=False
) ELSE (
    SET UserConfig.SVNAllowWorkingCopyUpdate=True
)
CALL :ClearBuffer
GOTO :SettingsSVN_Menu



REM # =============================================================================================
REM # Documentation: This will allow the program to fetch the revision changelog history.
REM # =============================================================================================
:SettingsSVN_ToggleFetchRevisionLog
IF %UserConfig.SVNAllowFetchRevisionLog% EQU True (
    SET UserConfig.SVNAllowFetchRevisionLog=False
) ELSE (
    SET UserConfig.SVNAllowFetchRevisionLog=True
)
CALL :ClearBuffer
GOTO :SettingsSVN_Menu



REM # =============================================================================================
REM # Documentation: This will allow the program to fetch the revision changelog history in an XML advanced formatting.
REM # =============================================================================================
:SettingsSVN_ToggleFetchRevisionLogXML
IF %UserConfig.SVNAllowFetchRevisionLogXML% EQU True (
    SET UserConfig.SVNAllowFetchRevisionLogXML=False
) ELSE (
    SET UserConfig.SVNAllowFetchRevisionLogXML=True
)
CALL :ClearBuffer
GOTO :SettingsSVN_Menu



REM ============================================================



REM # =============================================================================================
REM # Documentation: 7Zip Settings Menu
REM # =============================================================================================
:Settings7Zip_Menu
CALL :DashboardOrClassicalDisplay
ECHO 7Zip Control Panel
ECHO %Separator%
ECHO.
ECHO [1] 7Zip Master Control
ECHO     When set to 'True', this will allow the program to utilize 7Zip features and functionality.
ECHO     - Current Value: [%UserConfig.7ZipMasterControl%]
ECHO     - Detected: [%Detect_SVN%]
ECHO.
ECHO [2] Generate project archive builds
ECHO     This can allow the freshly compiled project to be compacted into a archive file that will be generated by the use of 7zip.
ECHO     - Current Value: [%UserConfig.7ZipGenerateArchive%]
ECHO.
ECHO [X] Exit
CALL :UserInput
GOTO :Settings7Zip_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:Settings7Zip_UserInput
IF "%STDIN%" EQU "1" GOTO :Settings7Zip_ToggleMaster
IF "%STDIN%" EQU "2" GOTO :Settings7Zip_ToggleGenerateArchive
IF /I "%STDIN%" EQU "X" GOTO :EOF
IF /I "%STDIN%" EQU "Exit" GOTO :EOF
CALL :BadInput& GOTO :Settings7Zip_Menu



REM # =============================================================================================
REM # Documentation: Allow the program to use 7Zip?  If this is set to false, the program is _NOT_ allowed to use 7Zip.
REM # =============================================================================================
:Settings7Zip_ToggleMaster
IF %UserConfig.7ZipMasterControl% EQU True (
    SET UserConfig.7ZipMasterControl=False
) ELSE (
    SET UserConfig.7ZipMasterControl=True
)
CALL :ClearBuffer
GOTO :Settings7Zip_Menu



REM # =============================================================================================
REM # Documentation: Allow the program to create an archive?
REM # =============================================================================================
:Settings7Zip_ToggleGenerateArchive
IF %UserConfig.7ZipGenerateArchive% EQU True (
    SET UserConfig.7ZipGenerateArchive=False
) ELSE (
    SET UserConfig.7ZipGenerateArchive=True
)
CALL :ClearBuffer
GOTO :Settings7Zip_Menu



REM ============================================================



REM # =============================================================================================
REM # Documentation: Inno Setup Builder Settings Menu
REM # =============================================================================================
:SettingsInno_Menu
CALL :DashboardOrClassicalDisplay
ECHO Inno Setup Control Panel
ECHO %Separator%
ECHO.
ECHO [1] Inno Master Control
ECHO     When set to 'True', this will allow the program to utilize Inno features and functionality.
ECHO     - Current Value: [%UserConfig.InnoMasterControl%]
ECHO     - Detected: [%Detect_InnoSetup%]
ECHO.
ECHO [2] Generate project setup executable
ECHO     This can allow the freshly compiled project to be compact within a setup file.  Thus, this allows anyone to easily install the project by using this setup file, which most software comes with today for Windows OS's.
ECHO     - Current Value: [%UserConfig.InnoGenerateSetup%]
ECHO.
ECHO [X] Exit
CALL :UserInput
GOTO :SettingsInno_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:SettingsInno_UserInput
IF "%STDIN%" EQU "1" GOTO :SettingsInno_ToggleMaster
IF "%STDIN%" EQU "2" GOTO :SettingsInno_GenerateSetup
IF /I "%STDIN%" EQU "X" GOTO :EOF
IF /I "%STDIN%" EQU "Exit" GOTO :EOF
CALL :BadInput& GOTO :SettingsInno_Menu



REM # =============================================================================================
REM # Documentation: Allow the program to use Inno?  If this is set to false, the program is _NOT_ allowed to use Inno Setup Builder.
REM # =============================================================================================
:SettingsInno_ToggleMaster
IF %UserConfig.InnoMasterControl% EQU True (
    SET UserConfig.InnoMasterControl=False
) ELSE (
    SET UserConfig.InnoMasterControl=True
)
CALL :ClearBuffer
GOTO :SettingsInno_Menu



REM # =============================================================================================
REM # Documentation: Allow the program to create an setup package?
REM # =============================================================================================
:SettingsInno_GenerateSetup
IF %UserConfig.InnoGenerateSetup% EQU True (
    SET UserConfig.InnoGenerateSetup=False
) ELSE (
    SET UserConfig.InnoGenerateSetup=True
)
CALL :ClearBuffer
GOTO :SettingsInno_Menu



REM ============================================================



REM # =============================================================================================
REM # Documentation: Visual Studio Settings Menu
REM # =============================================================================================
:SettingsVisualStudio_Menu
CALL :DashboardOrClassicalDisplay
CALL :VisualStudio_FindNiceName
ECHO Visual Studio Control Panel
ECHO %Separator%
ECHO.
ECHO Currently using: [%VisualStudioNiceName%]
ECHO.
ECHO Select Visual Studio Environment
ECHO [1] Visual Studio 2008
ECHO     - Detected: [%Detect_MSVS90COMMTOOLS%]
ECHO.
ECHO [2] Visual Studio 2010
ECHO     - Detected: [%Detect_MSVS100COMMTOOLS%]
ECHO.
ECHO [3] Visual Studio 2012
ECHO     - Detected: [%Detect_MSVS110COMMTOOLS%]
ECHO.
ECHO [4] Visual Studio 2013
ECHO     - Detected: [%Detect_MSVS120COMMTOOLS%]
ECHO.
ECHO [Auto] Automatically determine best version
ECHO     This will try to automatically try to find the best Visual Studio environment version for the %ProjectName% project.
ECHO.
ECHO [X] Exit
CALL :UserInput
GOTO :SettingsVisualStudio_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:SettingsVisualStudio_UserInput
IF "%STDIN%" EQU "1" GOTO :SettingsVisualStudio_Toggle2008
IF "%STDIN%" EQU "2" GOTO :SettingsVisualStudio_Toggle2010
IF "%STDIN%" EQU "3" GOTO :SettingsVisualStudio_Toggle2012
IF "%STDIN%" EQU "4" GOTO :SettingsVisualStudio_Toggle2013
IF /I "%STDIN%" EQU "Auto" GOTO :SettingsVisualStudio_Automatic
IF /I "%STDIN%" EQU "X" GOTO :EOF
IF /I "%STDIN%" EQU "Exit" GOTO :EOF
CALL :BadInput& GOTO :SettingsVisualStudio_Menu



REM # =============================================================================================
REM # Documentation: Select Visual Studio 2008 as the primary compiler, if possible.
REM # =============================================================================================
:SettingsVisualStudio_Toggle2008
IF %Detect_MSVS90COMMTOOLS% EQU True IF "%VisualStudio%" NEQ "%PathMSVS2008%" SET "VisualStudio=%PathMSVS2008%"
CALL :ClearBuffer
GOTO :SettingsVisualStudio_Menu



REM # =============================================================================================
REM # Documentation: Select Visual Studio 2010 as the primary compiler, if possible.
REM # =============================================================================================
:SettingsVisualStudio_Toggle2010
IF %Detect_MSVS100COMMTOOLS% EQU True IF "%VisualStudio%" NEQ "%PathMSVS2010%" SET "VisualStudio=%PathMSVS2010%"
CALL :ClearBuffer
GOTO :SettingsVisualStudio_Menu



REM # =============================================================================================
REM # Documentation: Select Visual Studio 2012 as the primary compiler, if possible.
REM # =============================================================================================
:SettingsVisualStudio_Toggle2012
IF %Detect_MSVS110COMMTOOLS% EQU True IF "%VisualStudio%" NEQ "%PathMSVS2012%" SET "VisualStudio=%PathMSVS2012%"
CALL :ClearBuffer
GOTO :SettingsVisualStudio_Menu



REM # =============================================================================================
REM # Documentation: Select Visual Studio 2013 as the primary compiler, if possible.
REM # =============================================================================================
:SettingsVisualStudio_Toggle2013
IF %Detect_MSVS120COMMTOOLS% EQU True IF "%VisualStudio%" NEQ "%PathMSVS2013%" SET "VisualStudio=%PathMSVS2013%"
CALL :ClearBuffer
GOTO :SettingsVisualStudio_Menu



REM # =============================================================================================
REM # Documentation: Allow the program to automatically determine the best Visual Studio version.
REM # =============================================================================================
:SettingsVisualStudio_Automatic
CALL :VisualStudio_AutomaticDetection
CALL :ClearBuffer
CALL :SettingsVisualStudio_Menu



REM ============================================================



REM # =============================================================================================
REM # Documentation: Directory Management Settings Menu
REM # =============================================================================================
:SettingsDirectoryManagement_Menu
CALL :DashboardOrClassicalDisplay
ECHO Directory Control Panel
ECHO %Separator%
ECHO.
ECHO [1] Locate %ProjectName% Directory
ECHO     This defines where the program can locate the %ProjectName% directory.
ECHO     This is an important setting that must be properly defined, otherwise this program will not work correctly.
REM pass through the detection
CALL :DetectionProject "%UserConfig.DirProjectWorkingCopy%"
REM ----
REM Capture the string for the detection
CALL :ExitCodeIntToStringCommon %ERRORLEVEL%
ECHO     - Detected: [%ProcessVarA%]
REM ----
ECHO     - Current Value: [%UserConfig.DirProjectWorkingCopy%]
ECHO.
ECHO [2] Keep %ProjectName%'s Setup directory contents
ECHO     The Setup directory usually contains some necessary prerequisites, such as .NET foundation, and SlimDX for example.
ECHO     - Current Value: [%UserConfig.KeepSetupDir%]
ECHO.
ECHO [3] Duplicate the compiled data from %ProjectName% directories to the module's local directory.
ECHO     When True, this will mirror all of the compiled data from the %ProjectName% Build directory to the compiled directory stored in this program.
ECHO        Stored compiled data in this program can be located at:
ECHO          [ %LocalDirectory.Builds% ]
ECHO     - Current Value: [%UserConfig.MirrorCompiledData%]
ECHO.
ECHO [X] Exit
CALL :UserInput
GOTO :SettingsDirectoryManagement_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the users input.
REM # =============================================================================================
:SettingsDirectoryManagement_UserInput
IF "%STDIN%" EQU "1" GOTO :SettingsDirectoryManagement_TargetProjectDirectory
IF "%STDIN%" EQU "2" GOTO :SettingsDirectoryManagement_ToggleSetupDir
IF "%STDIN%" EQU "3" GOTO :SettingsDirectoryManagement_ToggleFlushCompiledData
IF /I "%STDIN%" EQU "X" GOTO :EOF
IF /I "%STDIN%" EQU "Exit" GOTO :EOF
CALL :BadInput& GOTO :SettingsDirectoryManagement_Menu



REM # =============================================================================================
REM # Documentation: This function helps the user to safely set the target path of the project.
REM #   Keep in mind, that the target path must point to the directory that contains 'builder.sln' file, otherwise, nothing can be set properly.
REM # =============================================================================================
:SettingsDirectoryManagement_TargetProjectDirectory
CALL :ClearBuffer
CALL :DashboardOrClassicalDisplay
ECHO Target Path: %ProjectName%
ECHO %Separator%
ECHO.
ECHO This setting is very important - as this will help the program to successfully detect where the %ProjectName% project can be located.
ECHO.
ECHO Current Target:
ECHO %UserConfig.DirProjectWorkingCopy%
CALL :DetectionProject "%UserConfig.DirProjectWorkingCopy%"
REM Capture the string for the detection
CALL :ExitCodeIntToStringCommon %ERRORLEVEL%
ECHO Detected: [%ProcessVarA%]
ECHO.
ECHO.
ECHO Enter a new target:
ECHO.
ECHO Other options:
ECHO [X] Exit
CALL :UserInput
REM If the user opts out, leave this function now.
IF /I "%STDIN%" EQU "X" CALL :ClearBuffer& GOTO :SettingsDirectoryManagement_Menu
CALL :SettingsDirectoryManagement_TargetProjectDirectoryManageUpdate-Filter
CALL :DetectionProject "%STDIN%"
CALL :SettingsDirectoryManagement_TargetProjectDirectoryManageUpdate %ERRORLEVEL%
IF %ERRORLEVEL% EQU 0 CALL :Initialization_Driver UpdateProjectTargets
REM -----
REM If the new target exists, then we're done.
IF %ERRORLEVEL% EQU 0 CALL :ClearBuffer& GOTO :SettingsDirectoryManagement_Menu
REM This function is already big enough
CALL :SettingsDirectoryManagement_TargetProjectDirectoryBadLocation
CALL :ClearBuffer&
GOTO :SettingsDirectoryManagement_TargetProjectDirectory



REM # =============================================================================================
REM # Documentation: If the specified path does not exist when the user is attempting to update the project's location within the working copy, this error message will be displayed on the screen.
REM # =============================================================================================
:SettingsDirectoryManagement_TargetProjectDirectoryBadLocation
ECHO.&ECHO.
ECHO ^<!^>       Bad Target       ^<!^>
ECHO %SeparatorLong%
ECHO.
ECHO !ERR: The location specified does not exist or the 'Builder.sln' file was not found.
ECHO.
ECHO Directory contains:
ECHO %STDIN%
ECHO %SeparatorLong%
DIR /B "%STDIN%"
ECHO %SeparatorLong%
ECHO.
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Update the project's Working Copy target.
REM # =============================================================================================
:SettingsDirectoryManagement_TargetProjectDirectoryManageUpdate
IF %1 EQU 1 (
    SET "UserConfig.DirProjectWorkingCopy=%STDIN%"
    SET Detect_ProjectCore=True
    EXIT /B 0
) ELSE (
    REM The target location does not exist or is not what we are wanting
    EXIT /B 1
)



REM # =============================================================================================
REM # Documentation: Filter any trailing back-slashes.  If incase the user adds '\' at the ending of the string, remove it as this can cause issues with some of the programs like Subversion, for example.
REM # =============================================================================================
:SettingsDirectoryManagement_TargetProjectDirectoryManageUpdate-Filter
REM Using the variable value, read the last character from the value.  -1 == One character from the right side
IF "%STDIN:~-1%" EQU "\" (
    REM Select the range, the -1 means we are limiting the right side; we're effectively dropping 'one' character.
    SET "STDIN=%STDIN:~0,-1%"
) ELSE (
    GOTO :EOF
)
REM If incase the user adds more of the back slashes for whatever reason, call the function again until it is fixed.
GOTO :SettingsDirectoryManagement_TargetProjectDirectoryManageUpdate-Filter



REM # =============================================================================================
REM # Documentation: Keep the Setup directory, which contains Doom Builder 2 based prerequisites like SlimDX and .NET 3.5 installation packages?
REM # =============================================================================================
:SettingsDirectoryManagement_ToggleSetupDir
IF %UserConfig.KeepSetupDir% EQU True (
    SET UserConfig.KeepSetupDir=False
) ELSE (
    SET UserConfig.KeepSetupDir=True
)
CALL :ClearBuffer
GOTO :SettingsDirectoryManagement_Menu



REM # =============================================================================================
REM # Documentation: Mirror the compiled data that is stored in the Build directory to the local directory.
REM # =============================================================================================
:SettingsDirectoryManagement_ToggleFlushCompiledData
IF %UserConfig.MirrorCompiledData% EQU True (
    SET UserConfig.MirrorCompiledData=False
) ELSE (
    SET UserConfig.MirrorCompiledData=True
)
CALL :ClearBuffer
GOTO :SettingsDirectoryManagement_Menu




REM =====================================================================
REM User Configuration [Key Chain]
REM ----------------------------
REM This allows the user to effectively to update, create, and load their own configuration presets.
REM
REM This is compatible with Bootless Star's 1.2 Key Chain Model
REM =====================================================================


REM # =============================================================================================
REM # Parameters: [{int} Mode]
REM # Documentation: This driver function assures that the users configuration file is either loaded or saved properly.
REM # =============================================================================================
:UserPreset_Driver
REM First, check to make sure that we're not using the coreDefault configuration.  If the user is using the default, then opt out of this driver -- we can not use this feature.
IF %UserConfigurationKeyChainToken% EQU 0 EXIT /B 1

REM Cache the filename to a variable - for simplicity.
CALL :UserPreset_CacheFileName

REM Check the parameters and determine how to execute.
IF %1 EQU 0 CALL :UserPreset_Load "%ProcessVarA%"
IF %1 EQU 1 CALL :UserPreset_Update "%ProcessVarA%"
IF %1 EQU 2 CALL :UserPreset_Create "%ProcessVarA%"
REM Successful operation
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Cache the filename to a variable - for simplicity.
REM # =============================================================================================
:UserPreset_CacheFileName
SET "ProcessVarA=%LocalDirectory.UserConfig%\%UserConfigurationKeyChainToken%.bat"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} File]
REM # Documentation: Load the preset configuration file.
REM # =============================================================================================
:UserPreset_Load
IF EXIST "%~1" CALL "%~1"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} File]
REM # Documentation: Update the configuration file with newer settings.
REM # =============================================================================================
:UserPreset_Update
CALL :UserPreset_Create "%~1"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} File]
REM # Documentation: Create a new configuration file for this module.  This function is designed to create a new preset script file.  If in case a preset exists, this will automatically thrash it and write a new file within the filesystem.
REM # =============================================================================================
:UserPreset_Create
CALL :UserPreset_MakeFile_Header "%~1"
CALL :Initialization_Driver SaveUserConfig "%~1"
CALL :UserPreset_MakeFile_Footer "%~1"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} File]
REM # Documentation: This function will create a header for the configuration file; this supplies identity of the file.
REM # =============================================================================================
:UserPreset_MakeFile_Header
(ECHO REM Preset Configuration File)> "%~1"
(ECHO REM %ProjectName% version %ProjectVersion%)>> "%~1"
(ECHO REM Created on: %DATE% - %TIME%)>> "%~1"
(ECHO REM Filename: %UserConfigurationLoaded% - Key Chain: %UserConfigurationKeyChainToken%)>> "%~1"
(ECHO REM %SeparatorLong%)>> "%~1"
(ECHO.)>> "%~1"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{String} File]
REM # Documentation: This function will create a footer for the configuration file.
REM # =============================================================================================
:UserPreset_MakeFile_Footer
(ECHO GOTO :EOF)>> "%~1"
GOTO :EOF




REM =====================================================================
REM View Directories
REM ----------------------------
REM Within this section, this will allow the user to simply open directories right from the GUI shell -- nothing fancy other than EXPLORER calls.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: What directory does the user wish to view?
REM # =============================================================================================
:ViewDirectories
CALL :DashboardOrClassicalDisplay
ECHO Open a Directory Menu
ECHO %Separator%
ECHO.
ECHO [1] Regular builds directory
ECHO [2] Compressed directory
ECHO [3] Setup directory
ECHO [4] Log directory
ECHO [X] Exit
CALL :UserInput
GOTO :ViewDirectories_UserInput



REM # =============================================================================================
REM # Documentation: Inspect the user's input
REM # =============================================================================================
:ViewDirectories_UserInput
IF "%STDIN%" EQU "1" (
    CALL :ViewDirectories_InputDriver Builds
    GOTO :ViewDirectories
)
IF "%STDIN%" EQU "2" (
    CALL :ViewDirectories_InputDriver Archives
    GOTO :ViewDirectories
)
IF "%STDIN%" EQU "3" (
    CALL :ViewDirectories_InputDriver Setup
    GOTO :ViewDirectories
)
IF "%STDIN%" EQU "4" (
    CALL :ViewDirectories_InputDriver Log
    GOTO :ViewDirectories
)
IF /I "%STDIN%" EQU "X" GOTO :EOF
CALL :BadInput& GOTO :ViewDirectories



REM # =============================================================================================
REM # Documentation: Manage the users request in a simple package and slightly easier to manage.
REM # =============================================================================================
:ViewDirectories_InputDriver
IF %1 EQU Builds CALL :ViewDirectories_BuildsDir
IF %1 EQU Archives CALL :ViewDirectories_ArchivesDir
IF %1 EQU Setup CALL :ViewDirectories_SetupDir
IF %1 EQU Log CALL :ViewDirectories_LogDir
CALL :ClearBuffer
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Open the Builds Directory within the graphical user interface.
REM # =============================================================================================
:ViewDirectories_BuildsDir
CALL :Call_WindowsExplorer "%LocalDirectory.Builds%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Open the Archives Directory within the graphical user interface.
REM # =============================================================================================
:ViewDirectories_ArchivesDir
CALL :Call_WindowsExplorer "%LocalDirectory.Archives%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Open the Setup Build Directory within the graphical user interface.
REM # =============================================================================================
:ViewDirectories_SetupDir
CALL :Call_WindowsExplorer "%LocalDirectory.Setup%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Open the Log Directory within the graphical user interface.
REM # =============================================================================================
:ViewDirectories_LogDir
CALL :Call_WindowsExplorer "%LocalDirectory.Logs%"
GOTO :EOF




REM =====================================================================
REM Main Compiling Driver
REM ----------------------------
REM This is a simple driver for calling other functions that properly helps to compile the main project.
REM The CompileManager_Driver may seem ugly, but with Batch, readability is a bit - limited....
REM =====================================================================



REM # =============================================================================================
REM # Parameters: [{int} Plugins]
REM # Documentation: This driver will guide the entire compiling procedure.
REM #   Parameters: Plugins: 0=None, 1=Default, 2=All
REM # =============================================================================================
:CompileManager_Driver
CALL :DashboardOrClassicalDisplay

REM Display the header message
CALL :CompileProject_DisplayHeaderMessage

REM Check to make sure that all prerequisites and dependencies are available and ready to be used.  This detection is important, otherwise - everything will be a waste of time if some minor issue occurs.
CALL :CompileProject_CheckResources || GOTO :CompileProject_TerminateWithError


REM ----
REM Update the local working copy to the latest revision
CALL :CompileProject_SVNUpdateProject || GOTO :CompileProject_TerminateWithError


REM ----
REM Find the revision number
CALL :CompileProject_FetchSVNRevisionID || GOTO :CompileProject_TerminateWithError


REM ----
REM Update the project version ID; this is displayed when viewing the 'Help:About' dialog box in the project.
CALL :CompileProject_UpdateInternalProjectVersion || GOTO :CompileProject_TerminateWithError


REM ----
REM Update the file name for the projects output
CALL :CompileProject_GenerateProjectName || GOTO :CompileProject_TerminateWithError


REM ----
REM Load the Visual Studio environment within the session
CALL :CompileProject_MicrosoftVisualStudioEnvironment || GOTO :CompileProject_TerminateWithError


REM ----
REM Compile the core engine of the project.
CALL :CompileProject_CompileCoreEngine || GOTO :CompileProject_TerminateWithError


REM ----
REM Compile the help documentation for the project
CALL :CompileProject_CompileHelpDocumentation || GOTO :CompileProject_TerminateWithError


REM ----
REM Compile the plugins for the project
CALL :CompileProject_CompilePlugins %1 || GOTO :CompileProject_TerminateWithError


REM ----
REM Cleanup the directories
CALL :CompileProject_CleanupCompiledData || GOTO :CompileProject_TerminateWithError


REM ----
REM Fetch the revision log history
CALL :CompileProject_FetchSVNRevisionLogHistory || GOTO :CompileProject_TerminateWithError


REM ----
REM Record the revision to the ASCII file
CALL :CompileProject_SVNUpdateCachedRevision || GOTO :CompileProject_TerminateWithError


REM ----
REM Move the compiled project to the local directory
CALL :CompileProject_ProjectLocalDirectory || GOTO :CompileProject_TerminateWithError


REM ----
REM Place the compiled data in an archive data file
CALL :CompileProject_7ZipCompactData || GOTO :CompileProject_TerminateWithError


REM ----
REM Verify that contents within the archive
CALL :CompileProject_7ZipVerifyData || GOTO :CompileProject_TerminateWithError


REM ----
REM Generate a setup build using Inno
CALL :CompileProject_GenerateSetup || GOTO :CompileProject_TerminateWithError


REM ----
REM Cleanup the local working directory
CALL :CompileProject_CleanupWorkingCopy || GOTO :CompileProject_TerminateWithError


REM ----
REM Pop-up windows of the new builds
CALL :CompileProject_PopupWindowsExplorer || GOTO :CompileProject_TerminateWithError


REM ----
GOTO :CompileProject_TerminateSuccessfully




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                               7Zip


REM # =============================================================================================
REM # Documentation: Compact the files with 7zip?
REM # =============================================================================================
:CompileProject_7ZipCompactData
REM ----
REM Run this function?
IF %Detect_7Zip% EQU False EXIT /B 0
IF %UserConfig.7ZipMasterControl% EQU False EXIT /B 0
REM Does the user want to create an archive?
IF UserConfig.7ZipGenerateArchive EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Compacting Data with 7Zip"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM Filename
CALL :CompileProject_7ZipCommonSet_NameFile
REM Does it already exist?
IF EXIST "%LocalDirectory.Archives%\%FileName_Archive%" CALL :CompileProject_7ZipCompactData_ThrashExisting || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_7ZipCompactData_CompactData || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: This function will gather the information that is needed and then compact the data.
REM # =============================================================================================
:CompileProject_7ZipCompactData_CompactData
CALL :CompileProject_Display_IncomingTaskSubLevel "Compacting the data"
CALL :CompileProject_7ZipCommonSet 1
SET TaskCaller_CallLong=START "Batch Fork: 7Zip Process Task" /B /WAIT /%PriorityGeneral% "%Path7Zip%" a "%ProcessVarA%" "%ProcessVarB%" %ProcessVarC%
SET TaskCaller_NiceProgramName=Seven Zip
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: If incase another archive exists with that exact same name, remove it.
REM # =============================================================================================
:CompileProject_7ZipCompactData_ThrashExisting
CALL :CompileProject_Display_IncomingTaskSubLevel "Thrashing an already existing file [%LocalDirectory.Archives%\%FileName_Archive%]"
REM If the archive already exists, remove it from the file system.
SET TaskCaller_NiceProgramName=Delete
SET TaskCaller_CallLong=DEL /F /Q "%LocalDirectory.Archives%\%FileName_Archive%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Update the FileName_Archive variable name; this will come in handy later in the procedure.
REM # =============================================================================================
:CompileProject_7ZipCommonSet_NameFile
SET FileName_Archive=%FileName%.%SevZipFileExtension%
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Set up the arguments required to invoke 7zip.
REM # Parameters: [{int} 7ZipMode]
REM #   [1] = Compact Mode; setup extra parameters for 7Zip
REM # =============================================================================================
:CompileProject_7ZipCommonSet
REM Output Dir and Filename
SET ProcessVarA=%LocalDirectory.Archives%\%FileName_Archive%
IF %1 EQU 1 (
    REM Folder to compact
    SET ProcessVarB=%ProjectDirectory.DirCompiledTarget%\*
    REM Parameters
    SET ProcessVarC=-mx=%SevZipCompressionPass% -mmt=%SevZipMultiThreadingCPU% -t%SevZipArchiveFormat% -w"%LocalDirectory.Temp%"
)
GOTO :EOF



REM -------------------------------
REM -------------------------------
REM Verify the contents within an archive


REM # =============================================================================================
REM # Documentation: Verify the archive contents.
REM # =============================================================================================
:CompileProject_7ZipVerifyData
REM ----
REM Run this function?
IF %Detect_7Zip% EQU False EXIT /B 0
IF %UserConfig.7ZipMasterControl% EQU False EXIT /B 0
REM Does the user want to verify the contents?
IF %SevZipVerify% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Verifying Data with 7Zip"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_7ZipVerifyData_Task || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



:CompileProject_7ZipVerifyData_Task
CALL :CompileProject_Display_IncomingTaskSubLevel "Testing the archive's integrity"
CALL :CompileProject_7ZipCommonSet 0
SET TaskCaller_CallLong=START "Batch Fork: 7Zip Process Task" /B /WAIT /%PriorityGeneral% "%Path7Zip%" t "%ProcessVarA%" -r
SET TaskCaller_NiceProgramName=Seven Zip
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                          Check Resources


REM # =============================================================================================
REM # Documentation: This function will make sure that the all of the resources that are necessary for this program are available and ready.  If there is _one_ issue, terminate the program - unless it is possible to _assume_ settings automatically.
REM # =============================================================================================
:CompileProject_CheckResources
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Running Self-Check"
REM Print on the screen that the program is trying to self-check
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM ----
REM Check if the project can be detected
CALL :CompileProject_CheckResources_ProjectTarget || EXIT /B 1
REM ----
REM Check if all of the targets exists
CALL :CompileProject_CheckResources_CheckTargets || EXIT /B 1
REM ----
REM Make sure that their is a place to output the compiled data and not removed in the process.
CALL :CompileProject_CheckOutput || EXIT /B 1
REM ----
REM Check to make sure that 7Zip was detected
CALL :CompileProject_CheckResources_7Zip || EXIT /B 1
REM ----
REM Check to make sure that Subversion CommandLine Tools was detected
CALL :CompileProject_CheckResources_SVN || EXIT /B 1
REM ----
REM Check to make sure that Inno Setup Builder was detected
CALL :CompileProject_CheckResources_Inno || EXIT /B 1
REM ----
REM Check to make sure that the resources from Microsoft Visual Studio still exists
CALL :CompileProject_CheckResources_VisualStudio || EXIT /B 1
REM ----
REM Print the footer in the log.
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



:CompileProject_CheckResources_ProjectTarget
REM Were we able to find the projects directory?
CALL :CompileProject_Display_IncomingTaskSubLevel "Checking if %ProjectName%'s directory exists"
IF %Detect_ProjectCore% EQU False (
    REM We can't find the project
    CALL :CompileProject_CheckResources_ProjectTarget_ERR
    EXIT /B 1
)
REM We can find the project
EXIT /B 0



:CompileProject_CheckResources_ProjectTarget_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckResources_ProjectTarget_ERRLog
ECHO !ERR_CRIT!: Could not locate the %ProjectName% project!
ECHO This program can not continue until this issue has been resolved!
ECHO.
PAUSE
GOTO :EOF



:CompileProject_CheckResources_ProjectTarget_ERRLog
(ECHO !ERR_CRIT!: Could not locate the %ProjectName% project!) >> "%STDOUT%"
(ECHO This program can not continue until this issue has been resolved!) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



:CompileProject_CheckResources_CheckTargets
REM Can we find all of the targets?
CALL :CompileProject_Display_IncomingTaskSubLevel "Checking all of %ProjectName%'s resources and files"
REM VersionFromSVN Binary
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.VersionFromSVN%" || EXIT /B 1
REM ---
REM Build Directory
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.DirCompiledTarget%" || EXIT /B 1
REM ----
REM Help Directory
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.DirHelpTarget%" || EXIT /B 1
REM ----
REM Setup Directory
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.DirSetupTarget%" || EXIT /B 1
REM ----
REM Core Engine
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathCoreEngine%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathCoreAssembly%" || EXIT /B 1
REM ----
REM Builder Modes
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginBuilderModes%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginBuilderModesAssembly%" || EXIT /B 1
REM ----
REM Comments Panel
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginCommentsPanel%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginCommentsPanelAssembly%" || EXIT /B 1
REM ----
REM Copy Paste Sector Properties
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginCopyPaste%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginCopyPasteAssembly%" || EXIT /B 1
REM ----
REM Image Drawer
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginImageDraw%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginImageDrawAssembly%" || EXIT /B 1
REM ----
REM Statistics
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginStatistics%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginStatisticsAssembly%" || EXIT /B 1
REM ----
REM Tag Range
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginTagRange%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginTagRangeAssembly%" || EXIT /B 1
REM ----
REM USDF
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginUSDF%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginUSDFAssembly%" || EXIT /B 1
REM ----
REM GZDoom Virtual Mode
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginGZDoomVirtualMode%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginGZDoomVirtualModeAssembly%" || EXIT /B 1
REM ----
REM WADAuthor Virtual Mode
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginWadAuthorVirtualMode%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginWadAuthorVirtualModeAssembly%" || EXIT /B 1
REM ----
REM Visplane Explorer
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginVisplaneExplorerLibraryDir%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginVisplaneExplorerLibrary%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginVisplaneExplorerDir%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginVisplaneExplorer%" || EXIT /B 1
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginVisplaneExplorerAssembly%" || EXIT /B 1
REM ----
EXIT /B 0



:CompileProject_CheckResources_CheckTargets_Process
IF NOT EXIST "%~1" (
    CALL :CompileProject_CheckResources_CheckTargets_Process_ERR "%~1"
    EXIT /B 1
)
EXIT /B 0



REM # =============================================================================================
REM # Parameters: [{string} DefinedFile]
REM # Documentation: When a resource is not found within the filesystem, this function will display the error to the user.
REM # =============================================================================================
:CompileProject_CheckResources_CheckTargets_Process_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckResources_CheckTargets_Process_ERRLog "%~1"
ECHO !ERR_CRIT!: Could not find file:
ECHO   %~1
ECHO.
ECHO This program can not continue until this issue has been resolved!  Check for program updates or check to make sure that the location is properly defined.
ECHO.
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{string} DefinedFile]
REM # Documentation: When a resource is not found within the filesystem, this function will log the error.
REM # =============================================================================================
:CompileProject_CheckResources_CheckTargets_Process_ERRLog
(ECHO !ERR_CRIT!: Could not find file:) >> "%STDOUT%"
(ECHO   %~1) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
(ECHO This program can not continue until this issue has been resolved!  Check for program updates or check to make sure that the location is properly defined.) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Check to make sure that there _is_ going to be an output.
REM # =============================================================================================
:CompileProject_CheckOutput
CALL :CompileProject_Display_IncomingTaskSubLevel "Making sure that there is going to be an output"
CALL :CompileProject_CheckOutput_Scan
CALL :CompileProject_CheckOutput_CheckResults || (CALL :CompileProject_CheckOutput_Scan_ERR& EXIT /B 1)
EXIT /B 0



:CompileProject_CheckOutput_Scan
REM ----
REM Subversion
IF %Detect_SVN% EQU False (SET ProcessVarA=False)
IF %UserConfig.SVNMasterControl% EQU False (SET ProcessVarA=False)
IF %UserConfig.SVNAllowWorkingCopyRevert% EQU True (SET ProcessVarA=False)
REM ----
REM 7Zip
IF %Detect_7Zip% EQU False (SET ProcessVarB=False)
IF %UserConfig.7ZipMasterControl% EQU False (SET ProcessVarB=False)
IF %UserConfig.7ZipGenerateArchive% EQU False (SET ProcessVarB=False)
REM ----
REM Inno Setup
IF %Detect_InnoSetup% EQU False (SET ProcessVarC=False)
IF %UserConfig.InnoMasterControl% EQU False (SET ProcessVarC=False)
IF %UserConfig.InnoGenerateSetup% EQU False (SET ProcessVarC=False)
REM ----
REM Mirror
IF %UserConfig.MirrorCompiledData% EQU False (SET ProcessVarD=False)
GOTO :EOF



:CompileProject_CheckOutput_CheckResults
REM If everything is false, then there's a potential error.
IF %ProcessVarA% EQU False IF %ProcessVarB% EQU False IF %ProcessVarC% EQU False IF %ProcessVarD% EQU False EXIT /B 1
EXIT /B 0



:CompileProject_CheckOutput_Scan_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckOutput_Scan_ERRLog
ECHO !ERR_CRIT!: There is no proper output!
ECHO   After the program has compiled the %ProjectName% project, it will not be available for the user as it will be overwritten in the SVN directory.  Please review the settings and make sure that the compiled output will be either mirrored or in an compact file.
ECHO.
PAUSE
GOTO :EOF



:CompileProject_CheckOutput_Scan_ERRLog
(ECHO !ERR_CRIT!: There is no proper output!) >> "%STDOUT%"
(ECHO   After the program has compiled the %ProjectName% project, it will not be available for the user as it will be overwritten in the SVN directory.  Please review the settings and make sure that the compiled output will be either mirrored or in an compact file.) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Check to make sure that the 7Zip software was detected from the core.
REM # =============================================================================================
:CompileProject_CheckResources_7Zip
CALL :CompileProject_Display_IncomingTaskSubLevel "Checking if 7Zip was detected"
REM Check to see if the user toggled the Master Switch
CALL :CompileProject_CheckResources_CheckMasterSwitch %UserConfig.7ZipMasterControl% || EXIT /B 0
IF %Detect_7Zip% EQU False (
    REM We couldn't find the 7Zip software
    CALL :CompileProject_CheckResources_7Zip_ERR
    EXIT /B 1
)
REM Detected 7Zip
EXIT /B 0



:CompileProject_CheckResources_7Zip_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckResources_7Zip_ERRLog
ECHO !ERR_CRIT!: Could not find 7Zip software!
ECHO This program can not continue until this issue has been resolved!
ECHO.
PAUSE
GOTO :EOF



:CompileProject_CheckResources_7Zip_ERRLog
(ECHO !ERR_CRIT!: Could not find 7Zip software!) >> "%STDOUT%"
(ECHO This program can not continue until this issue has been resolved!) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Check to make sure that the SVN software was detected from the core.
REM # =============================================================================================
:CompileProject_CheckResources_SVN
CALL :CompileProject_Display_IncomingTaskSubLevel "Checking if Subversion CommandLine Tools was detected"
REM Check to see if the user toggled the Master Switch
CALL :CompileProject_CheckResources_CheckMasterSwitch %UserConfig.SVNMasterControl% || EXIT /B 0
IF %Detect_SVN% EQU False (
    REM We couldn't find the SVN software
    CALL :CompileProject_CheckResources_SVN_ERR
    EXIT /B 1
)
REM Detected SVN
EXIT /B 0



:CompileProject_CheckResources_SVN_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckResources_SVN_ERRLog
ECHO !ERR_CRIT!: Could not find Subversion CommandLine Tools!
ECHO This program can not continue until this issue has been resolved!
ECHO.
PAUSE
GOTO :EOF



:CompileProject_CheckResources_SVN_ERRLog
(ECHO !ERR_CRIT!: Could not find Subversion CommandLine Tools!) >> "%STDOUT%"
(ECHO This program can not continue until this issue has been resolved!) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Check to make sure that the Inno Setup Builder software was detected from the core.
REM # =============================================================================================
:CompileProject_CheckResources_Inno
CALL :CompileProject_Display_IncomingTaskSubLevel "Checking if Inno Setup Builder was detected"
REM Check to see if the user toggled the Master Switch
CALL :CompileProject_CheckResources_CheckMasterSwitch %UserConfig.InnoMasterControl% || EXIT /B 0
IF %Detect_InnoSetup% EQU False (
    REM We couldn't find the Inno Setup Builder software
    CALL :CompileProject_CheckResources_Inno_ERR
    EXIT /B 1
)
REM Detected Inno
EXIT /B 0



:CompileProject_CheckResources_Inno_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckResources_Inno_ERRLog
ECHO !ERR_CRIT!: Could not find Inno Setup Builder!
ECHO This program can not continue until this issue has been resolved!
ECHO.
PAUSE
GOTO :EOF



:CompileProject_CheckResources_Inno_ERRLog
(ECHO !ERR_CRIT!: Could not find Inno Setup Builder!) >> "%STDOUT%"
(ECHO This program can not continue until this issue has been resolved!) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Check to make sure that the Visual Studio resources still exists.
REM # =============================================================================================
:CompileProject_CheckResources_VisualStudio
CALL :CompileProject_Display_IncomingTaskSubLevel "Checking Microsoft Visual Studio resources are still available"

IF NOT EXIST "%VisualStudio%" (
    REM We couldn't find the resources from Visual Studio
    CALL :CompileProject_CheckResources_VisualStudio_ERR
    EXIT /B 1
)
REM Detected Visual Studio Resources
EXIT /B 0



:CompileProject_CheckResources_VisualStudio_ERR
IF %ToggleLog% EQU True CALL :CompileProject_CheckResources_VisualStudio_ERRLog
ECHO !ERR_CRIT!: Could not find the resources from Microsoft Visual Studio [ %VisualStudioNiceName% ]!
ECHO This program can not continue until this issue has been resolved!
ECHO.
PAUSE
GOTO :EOF



:CompileProject_CheckResources_VisualStudio_ERRLog
(ECHO !ERR_CRIT!: Could not find the resources from Microsoft Visual Studio [ %VisualStudioNiceName% ]!) >> "%STDOUT%"
(ECHO This program can not continue until this issue has been resolved!) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Parameters: [{Bool} MasterSwitch]
REM # Documentation: Check the master switch value and return it's value in a Return call.
REM # =============================================================================================
:CompileProject_CheckResources_CheckMasterSwitch
IF %1 EQU False (
    REM The user requested that the 
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Skipping; user requested."
    EXIT /B 1
) ELSE (
    EXIT /B 0
)




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                       Clean Up Local Build


REM # =============================================================================================
REM # Documentation: This function will clean up the local working copy and make sure that all of the superfluous files have been thrashed from the directories.
REM # =============================================================================================
:CompileProject_CleanupCompiledData
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Cleaning up"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM Clean up the Working Copy within the build directory
CALL :CompileProject_CleanupCompiledData_TaskGeneralCleanUp || (CALL :CaughtErrorSignal& EXIT /B 1)
REM Thrash the Setup directory located in the build directory at the Working Copy?
IF %UserConfig.KeepSetupDir% EQU False CALL :CompileProject_CleanupCompiledData_TaskWorkingCopyCleanUp || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B %ERRORLEVEL%



:CompileProject_CleanupCompiledData_TaskGeneralCleanUp
CALL :CompileProject_Display_IncomingTaskSubLevel "Thrashing superfluous data files that is not required for %ProjectName% engine"
SET TaskCaller_NiceProgramName=Delete
SET TaskCaller_CallLong=DEL /F /Q "%ProjectDirectory.DirCompiledTarget%\*.pdb" "%ProjectDirectory.DirCompiledTarget%\Builder.xml" "%ProjectDirectory.DirCompiledTarget%\Plugins\*.pdb" "%ProjectDirectory.DirCompiledTarget%\Plugins\Builder.exe" "%ProjectDirectory.DirCompiledTarget%\Plugins\Builder.xml" "%ProjectDirectory.DirCompiledTarget%\Plugins\Sharpzip.dll" "%ProjectDirectory.DirCompiledTarget%\Plugins\TrackBar.dll" "%ProjectDirectory.DirCompiledTarget%\Plugins\SlimDX.dll"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Expunge the Setup directory that is located within the Standard Output folder.
REM # =============================================================================================
:CompileProject_CleanupCompiledData_TaskWorkingCopyCleanUp
REM Safety net: Incase if the user _wants_ a setup package, then the Setup directory _must_ be present!
IF %UserConfig.InnoGenerateSetup% EQU True (
    IF %UserConfig.InnoMasterControl% EQU True GOTO :CompileProject_CleanupCompiledData_TaskWorkingCopyCleanUp_Required
)
CALL :CompileProject_CleanupCompiledData_TaskWorkingCopyCleanUp_Task
EXIT /B %ERRORLEVEL%



:CompileProject_CleanupCompiledData_TaskWorkingCopyCleanUp_Required
REM If the directory is required, then this function will notify the user that the request can not be satisfied, but the program will continue to operate regardless.
CALL :CompileProject_Display_IncomingTaskSubLevelMSG "!ERR: Can not remove %ProjectDirectory.DirCompileSetupTarget% because Inno Setup Builder requires this directory!"
EXIT /B 0



:CompileProject_CleanupCompiledData_TaskWorkingCopyCleanUp_Task
CALL :CompileProject_Display_IncomingTaskSubLevel "Thrashing the Setup Directory"
SET TaskCaller_NiceProgramName=Delete
SET TaskCaller_CallLong=RD /S /Q "%ProjectDirectory.DirCompileSetupTarget%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                       Clean Up SVN Local Working Copy



REM # =============================================================================================
REM # Documentation: Revert and clean up the Local Working Copy.
REM # =============================================================================================
:CompileProject_CleanupWorkingCopy
REM ----
REM Run this function?
IF %Detect_SVN% EQU False EXIT /B 0
IF %UserConfig.SVNMasterControl% EQU False EXIT /B 0
REM Does the user want the local working copy to be reverted?
IF %UserConfig.SVNAllowWorkingCopyRevert% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Cleaning up SVN Local Working Copy"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_CleanupWorkingCopy_ThrashBuildDir || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CleanupWorkingCopy_Cleanup || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CleanupWorkingCopy_Revert || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



:CompileProject_CleanupWorkingCopy_ThrashBuildDir
CALL :CompileProject_Display_IncomingTaskSubLevel "Expunging the Build directory from the %ProjectName% SVN Local Working Copy"
SET TaskCaller_CallLong=RD /S /Q "%ProjectDirectory.DirCompiledTarget%"
SET TaskCaller_NiceProgramName=Remove Directory
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CleanupWorkingCopy_Cleanup
CALL :CompileProject_Display_IncomingTaskSubLevel "Cleaning up the %ProjectName% SVN Local Working Copy"
SET TaskCaller_CallLong=SVN cleanup "%UserConfig.DirProjectWorkingCopy%"
SET TaskCaller_NiceProgramName=SVN
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CleanupWorkingCopy_Revert
CALL :CompileProject_Display_IncomingTaskSubLevel "Rolling-back the %ProjectName% SVN Local Working Copy"
SET TaskCaller_CallLong=SVN revert "%UserConfig.DirProjectWorkingCopy%" -R
SET TaskCaller_NiceProgramName=SVN
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                   Doom Builder 2 Core Software


REM # =============================================================================================
REM # Documentation: Compile the core engine of the project
REM # =============================================================================================
:CompileProject_CompileCoreEngine
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Compiling %ProjectName% core engine"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_CompileCoreEngine_UpdateVersion_Task || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompileCoreEngine_Task || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompileCoreEngine_FlagLargeAddress || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B %ERRORLEVEL%



:CompileProject_CompileCoreEngine_UpdateVersion_Task
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the %ProjectName% core engine version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathCoreAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompileCoreEngine_Task
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the %ProjectName% core engine"
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathCoreEngine%" %MSVSArg%
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompileCoreEngine_FlagLargeAddress
CALL :CompileProject_Display_IncomingTaskSubLevel "Issuing the Large Address flag to the %ProjectName% core engine"
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% "%VisualStudio%\..\..\VC\bin\editbin.exe" /LARGEADDRESSAWARE "%ProjectDirectory.PathCoreEngine%"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                 Doom Builder 2 Help Documentation


REM # =============================================================================================
REM # Documentation: Compile the help documentation for the project.
REM # =============================================================================================
:CompileProject_CompileHelpDocumentation
REM ----
REM Run this function?
IF %Detect_MSHTMLWorkShop% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Compiling %ProjectName% help documentation"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_CompileHelpDocumentation_Task || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B %ERRORLEVEL%



:CompileProject_CompileHelpDocumentation_Task
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the %ProjectName% help documentation"
SET TaskCaller_CallLong=START "Batch Fork: HTML Workshop Compiler Task" /B /WAIT /%PriorityGeneral% "%PathMSHTMLWorkShop%" "%ProjectDirectory.DirHelpTarget%\Refmanual.hhp"
SET TaskCaller_NiceProgramName=Microsoft HTML Workshop
CALL :CompileProject_TaskOperation
CALL :CompileProject_FlipBit_Common %ERRORLEVEL%
EXIT /B %ERRORLEVEL%




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                             File Name


REM # =============================================================================================
REM # Documentation: Determine the file and directory name of the project and the current revision number.
REM # =============================================================================================
:CompileProject_GenerateProjectName
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Generating project name"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_GenerateProjectName_MakeName
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Create a generalized name; it is possible for the user to not have a revision ID - which no revision number will be stored.
REM # =============================================================================================
:CompileProject_GenerateProjectName_MakeName
REM This name is used to compile the project with the revision number
REM  For example: Project-r0000
IF %SVNRevisionNew% EQU -1 (
    REM If we can not get the newer revision, then exclude the revision number in the name.
    SET FileName=%ProjectNameCompact%
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "The file naming scheme will not include a revision number."
) ELSE (
    SET FileName=%ProjectNameCompact%-r%SVNRevisionNew%
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "The file naming scheme will include a revision number."
)
GOTO :EOF




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                       General Program Use


REM # =============================================================================================
REM # Documentation: This function will help keep the activity messages in a cleaner standard.
REM # Parameters:
REM #   [string] TaskName
REM # =============================================================================================
:CompileProject_Display_IncomingTask
IF %ToggleLog% EQU True CALL :CompileProject_Display_IncomingTaskLog "%~1"
ECHO %~1. . .
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will help keep the activity messages in a cleaner standard.
REM # Parameters:
REM #   [string] TaskName
REM # =============================================================================================
:CompileProject_Display_IncomingTaskSubLevel
IF %ToggleLog% EQU True CALL :CompileProject_Display_IncomingTaskLog "%~1"
ECHO    %~1. . .
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will help keep the activity messages in a cleaner standard.  Messages passed through this function are only treated as standard messages and not fully procedures.
REM # Parameters:
REM #   [string] TaskName
REM # =============================================================================================
:CompileProject_Display_IncomingTaskSubLevelMSG
(ECHO %~1) >> "%STDOUT%"
ECHO    %~1
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will help keep the log activity messages in a cleaner standard.
REM # Parameters:
REM #   [string] TaskName
REM # =============================================================================================
:CompileProject_Display_IncomingTaskLog
(ECHO Performing Task:) >> "%STDOUT%"
(ECHO   %~1) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
(ECHO.) >> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Run the operation by invoking the defined variable 'TaskCaller_CallLong'.
REM # =============================================================================================
:CompileProject_TaskOperation
REM Run the operation that is packaged.
%TaskCaller_CallLong%>> "%STDOUT%" 2>&1
SET ExitCode=%ERRORLEVEL%
REM If we're logging, call the log-footer
IF %ToggleLog% EQU True CALL :CompileProject_LogFooter
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Always print this to log files after executing a task or a program.
REM # =============================================================================================
:CompileProject_LogFooter
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO Program Called:)>> "%STDOUT%"
(ECHO   %TaskCaller_NiceProgramName%)>> "%STDOUT%"
(ECHO Task Performed:)>> "%STDOUT%"
(ECHO   %TaskCaller_CallLong%)>> "%STDOUT%"
(ECHO Exit Code: %ExitCode%)>> "%STDOUT%"
(ECHO %Separator%)>> "%STDOUT%"
(ECHO Time: %Time%   -   Date: %Date%)>> "%STDOUT%"
(ECHO Project Name: %ProjectName%   -   Module Version: %ProjectVersion%)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Always print this to logfiles after a driver finished executing.
REM # Parameters: [{String} DriverMainProcess]
REM # =============================================================================================
:CompileProject_DriverLogFooter
REM ----
REM If incase the drivers call this function, opt out when the program is not logging.
IF %ToggleLog% EQU False (GOTO :EOF)
REM ----
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO %SeparatorSmall%)>> "%STDOUT%"
(ECHO %Separator%)>> "%STDOUT%"
(ECHO Finished Executing Task:)>> "%STDOUT%"
(ECHO   %~1)>> "%STDOUT%"
(ECHO Time: %Time%   -   Date: %Date%)>> "%STDOUT%"
(ECHO Project Name: %ProjectName%   -   Module Version: %ProjectVersion%)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO %Separator%)>> "%STDOUT%"
(ECHO %Separator%)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Cautionary note; the user should never close this program prematurely.
REM # =============================================================================================
:CompileProject_DisplayHeaderMessage
REM Just display text on the terminal
ECHO ^<i^>       Important       ^<i^>
ECHO Terminating this program prematurely can cause either hanging or zombie processes, in case can either hog the system's resources for an extended amount of time or will never terminate.
ECHO.
ECHO Compiling this project might take several minutes, please be patient!
ECHO.&ECHO.
GOTO :EOF



REM # =============================================================================================
REM # Documentation: For programs that uses reverse of the standard return code, this function will reverse them back to normal so the program will understand how to to properly treat the return value.
REM # =============================================================================================
:CompileProject_FlipBit_Common
IF %1 EQU 1 (
    REM 1 = Successful Operation
    EXIT /B 0
) ELSE (
    REM Failure or something else
    EXIT /B 1
)



REM --------------------
REM ====================




REM # =============================================================================================
REM # Documentation: An error occurred while compiling the project.
REM # =============================================================================================
:CompileProject_TerminateWithError
IF %UseBell% GEQ 1 ECHO %SND_BELL%& ECHO %SND_BELL%
ECHO ^<!!!^>       FATAL ERROR       ^<!!!^>
ECHO %SeparatorLong%
ECHO.
ECHO An error occurred while compiling the %ProjectName% project!
REM If the user is logging, tell them where they can find the logs
IF %ToggleLog% EQU True (
    ECHO Check the log for more information about the error.
    ECHO Log Directory:
    ECHO   %LocalDirectory.Logs%
    ECHO Log Name:
    ECHO   %STDOUT%
)
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Everything was compiled successfully.
REM # =============================================================================================
:CompileProject_TerminateSuccessfully
IF %UseBell% EQU 1 ECHO %SND_BELL%
ECHO ^<-^>       Operation Successful       ^<-^>
ECHO %SeparatorLong%
ECHO.
ECHO The %ProjectName% project has been compiled successfully!
PAUSE
GOTO :EOF




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                        Inno Setup Builder


REM # =============================================================================================
REM # Documentation: Create a Inno setup package.
REM # =============================================================================================
:CompileProject_GenerateSetup
REM ----
REM Run this function?
IF %Detect_InnoSetup% EQU False EXIT /B 0
IF %UserConfig.InnoMasterControl% EQU False EXIT /B 0
REM Does the user want to create a setup
IF %UserConfig.InnoGenerateSetup% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Generating Setup"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM Filename
CALL :CompileProject_GenerateSetup_NameFile
REM Does it already exist?
IF EXIST "%LocalDirectory.Setup%\%FileName_Setup%" CALL :CompileProject_GenerateSetup_ThrashExisting || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_GenerateSetup_CreateSetup || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_GenerateSetup_MoveSetupToLocal || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



:CompileProject_GenerateSetup_CreateSetup
CALL :CompileProject_Display_IncomingTaskSubLevel "Generating a setup for the %ProjectName% project"
SET TaskCaller_NiceProgramName=Inno Setup Builder
SET TaskCaller_CallLong=START "Batch Fork: Inno Setup Process Task" /B /WAIT /%PriorityGeneral% "%PathInnoSetup%" "%ProjectDirectory.DirSetupTarget%\builder2_setup.iss"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: If incase the previous setup package still exists, thrash it.
REM # =============================================================================================
:CompileProject_GenerateSetup_ThrashExisting
CALL :CompileProject_Display_IncomingTaskSubLevel "Thrashing an already existing file [%LocalDirectory.Setup%\%FileName_Setup%]"
REM If the package already exists, remove it from the file system.
SET TaskCaller_NiceProgramName=Delete
SET TaskCaller_CallLong=DEL /F /Q "%LocalDirectory.Setup%\%FileName_Setup%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%


REM # =============================================================================================
REM # Documentation: Transition the files from Doom Builder's Working Copy directory to Bootless Star's directory branch.
REM # =============================================================================================
:CompileProject_GenerateSetup_MoveSetupToLocal
CALL :CompileProject_Display_IncomingTaskSubLevel "Transitioning the setup file to the local directory"
SET TaskCaller_CallLong=COPY /V /Z "%ProjectDirectory.DirReleaseTarget%\builder2_setup.exe" "%LocalDirectory.Setup%\%FileName_Setup%"
SET TaskCaller_NiceProgramName=Copy
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Update the filename for future use.
REM # =============================================================================================
:CompileProject_GenerateSetup_NameFile
SET FileName_Setup=%FileName%_Setup.exe
GOTO :EOF




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                      Project Local Directory


REM # =============================================================================================
REM # Documentation: Determine the file and directory name of the project and the current revision number.
REM # =============================================================================================
:CompileProject_ProjectLocalDirectory
REM Did the user want to run this functionality?
IF %UserConfig.MirrorCompiledData% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Mirroring compiled data to Local Directory"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_GenerateProjectName_CheckDirectoryExists || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_GenerateProjectName_MakeDirectory || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_GenerateProjectName_Duplicate || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Create a directory which will store the output contents.
REM # =============================================================================================
:CompileProject_GenerateProjectName_MakeDirectory
REM Create the directory
CALL :CompileProject_Display_IncomingTaskSubLevel "Creating the %FileName% directory"
SET TaskCaller_CallLong=MKDIR %LocalDirectory.Builds%\%FileName%
SET TaskCaller_NiceProgramName=Make Directory
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Does the directory already exists?  If it does, let the user know that the directory is about to be thrashed in exchange for new content.
REM # =============================================================================================
:CompileProject_GenerateProjectName_CheckDirectoryExists
IF EXIST "%LocalDirectory.Builds%\%FileName%" CALL :CompileProject_GenerateProjectName_CheckDirectoryExists_PromptChoice || (CALL :CaughtErrorSignal& EXIT /B 1)
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Manage how the duplicated directory should be removed.
REM # =============================================================================================
:CompileProject_GenerateProjectName_CheckDirectoryExists_PromptChoice
CALL :CompileProject_GenerateProjectName_CheckDirectoryExists_PromptChoice_PrintMessage
CALL :CompileProject_GenerateProjectName_CheckDirectoryExists_PromptChoice_ExpungeDirectory || EXIT /B 1
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Display a message that lets the user know that a directory already exists and can not be used until the directory has been thrashed.
REM # =============================================================================================
:CompileProject_GenerateProjectName_CheckDirectoryExists_PromptChoice_PrintMessage
ECHO.&ECHO.
ECHO ^<?^>       Directory Already Exists       ^<?^>
ECHO %SeparatorLong%
ECHO.
ECHO Found an existing compiled build which is one step away from being permanently deleted.  Any personal files [.wad files for example] will be expunged!  Move any personal files to a safe location before continuing!
ECHO.
ECHO Location:
ECHO   %LocalDirectory.Builds%\%FileName%
ECHO.
PAUSE
ECHO.
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Expunge the duplicated directory and all of the contents inside within the directory.
REM # =============================================================================================
:CompileProject_GenerateProjectName_CheckDirectoryExists_PromptChoice_ExpungeDirectory
CALL :CompileProject_Display_IncomingTaskSubLevel "Thrashing the existing directory"
SET TaskCaller_CallLong=RMDIR /S /Q %LocalDirectory.Builds%\%FileName%
SET TaskCaller_NiceProgramName=Remove Directory and Files
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Mirror the compiled data that is located in the Build directory to the newly created directory in the local directory.
REM # =============================================================================================
:CompileProject_GenerateProjectName_Duplicate
CALL :CompileProject_Display_IncomingTaskSubLevel "Duplicating compiled data to the local directory"
SET TaskCaller_CallLong=XCOPY %XCopyArg%/E /I "%ProjectDirectory.DirCompiledTarget%" "%LocalDirectory.Builds%\%FileName%"
SET TaskCaller_NiceProgramName=XCOPY
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                  Internal Project Version Update


REM # =============================================================================================
REM # Documentation: Allow the program to update the software's version; this will include the working copy revision ID in the software's version ID.  For example, if the software's revision is 1983, then this revision ID can be used in the program by manipulating a file from the software's source code.
REM # =============================================================================================
:CompileProject_UpdateInternalProjectVersion
REM This is dependent on the CompileProject_FetchSVNRevisionID function.
REM ----
REM I know this is bad on the hardware level, but flip the variable false.
IF %ProjectModify.UpdateInternalProjectVersion% EQU True SET ProjectModify.UpdateInternalProjectVersion=False
REM Run this function?
IF %Detect_SVN% EQU False EXIT /B 0
IF %UserConfig.SVNMasterControl% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Determining if the %ProjectName% version can be updated"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM Check to see if the 'VersionFromSVN.exe' exists
IF NOT EXIST "%UserConfig.DirProjectWorkingCopy%\VersionFromSVN.exe" (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Can not update the %ProjectName% project's version."
    CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
    EXIT /B 0
)
REM ----
REM Too make editing this program a bit easier, the update is managed while compiling the project.  This function is merely going to flip an variable to allow the program to update the version.
SET ProjectModify.UpdateInternalProjectVersion=True
CALL :CompileProject_Display_IncomingTaskSubLevelMSG "It is possible to update the %ProjectName% project's version."
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                 Visual Studio Environment Control


REM # =============================================================================================
REM # Documentation: Load the Visual Studio environment within the current session.
REM # =============================================================================================
:CompileProject_MicrosoftVisualStudioEnvironment
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Calling Microsoft Visual Studio environment"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_MicrosoftVisualStudioEnvironment_ConfirmCheck || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_MicrosoftVisualStudioEnvironment_TaskVSLoad || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Load the environment
REM # =============================================================================================
:CompileProject_MicrosoftVisualStudioEnvironment_TaskVSLoad
CALL :CompileProject_Display_IncomingTaskSubLevel "Setting up the Microsoft Visual Studio environment"
SET TaskCaller_CallLong=CALL "%VisualStudio%\vsvars32.bat"
SET TaskCaller_NiceProgramName=Call Script or Function
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Check to make sure that the required file exists before executing the file.
REM # =============================================================================================
:CompileProject_MicrosoftVisualStudioEnvironment_ConfirmCheck
IF EXIST "%VisualStudio%" (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Detected Microsoft Visual Studio's tool set!"
    EXIT /B 0
) ELSE (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "!ERR: Could not detected Microsoft Visual Studio's tool set!"
    EXIT /B 1
)




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                      Doom Builder 2 Plugins


REM # =============================================================================================
REM # Parameters: [{int} TypeOfPlugins]
REM # Documentation: This driver will help make sure that the right plugins are compiled.
REM # =============================================================================================
:CompileProject_CompilePlugins
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Compiling selective plugins"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM No plugins
IF %1 EQU 0 CALL :CompileProject_CompilePlugins_NoPlugins || EXIT /B 1
REM Default plugins
IF %1 EQU 1 CALL :CompileProject_CompilePlugins_DefaultPlugins || EXIT /B 1
REM All plugins
IF %1 EQU 2 CALL :CompileProject_CompilePlugins_AllPlugins || EXIT /B 1
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Do not compile any of the plugins.
REM # =============================================================================================
:CompileProject_CompilePlugins_NoPlugins
CALL :CompileProject_Display_IncomingTaskSubLevelMSG "User requested to not compile any plugins!"
EXIT /B 0


REM # =============================================================================================
REM # Documentation: Compile only the default plugins; this is either determined by the developers of the project, or by 'MakeRelease.bat'.
REM # =============================================================================================
:CompileProject_CompilePlugins_DefaultPlugins
CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Compiling default plugins for the %ProjectName% project"
CALL :CompileProject_CompilePlugins_BuilderModes || (CALL :CaughtErrorSignal& EXIT /B 1)
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Compile all plugins that is known; Any supported plugin that is located in the plugins directory is fair game.
REM # =============================================================================================
:CompileProject_CompilePlugins_AllPlugins
CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Compiling all available plugins for the %ProjectName% project"
CALL :CompileProject_CompilePlugins_BuilderModes || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_CommentsPanel || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_CopyPaste || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_ImageDrawer || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_Statistics || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_TagRange || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_USDF || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_GZDoomVirtualMode || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_WadAuthorVirtualMode || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_CompilePlugins_VisplaneExplorer || (CALL :CaughtErrorSignal& EXIT /B 1)
EXIT /B 0




REM -------------------------------
REM -------------------------------
REM Builder Modes


REM # =============================================================================================
REM # Documentation: Compile the BuilderModes plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_BuilderModes
CALL :CompileProject_CompilePlugins_BuilderModes_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_BuilderModes_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_BuilderModes_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Builder Modes plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginBuilderModesAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_BuilderModes_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Builder Modes plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginBuilderModes%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Comments Panel


REM # =============================================================================================
REM # Documentation: Compile the Comments Panel plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_CommentsPanel
CALL :CompileProject_CompilePlugins_CommentsPanel_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_CommentsPanel_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_CommentsPanel_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Comments Panel plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginCommentsPanelAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_CommentsPanel_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Comments Panel plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginCommentsPanel%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Copy Paste


REM # =============================================================================================
REM # Documentation: Compile the Copy+Paste plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_CopyPaste
CALL :CompileProject_CompilePlugins_CopyPaste_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_CopyPaste_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_CopyPaste_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Sector Copy and Paste Properties plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginCopyPasteAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_CopyPaste_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Sector Copy and Paste Properties plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginCopyPaste%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Image Drawer


REM # =============================================================================================
REM # Documentation: Compile the Image Drawer plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_ImageDrawer
CALL :CompileProject_CompilePlugins_ImageDrawer_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_ImageDrawer_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_ImageDrawer_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Image Drawer plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginImageDrawAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%


:CompileProject_CompilePlugins_ImageDrawer_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Image Drawer plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginImageDraw%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Statistics


REM # =============================================================================================
REM # Documentation: Compile the Statistics plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_Statistics
CALL :CompileProject_CompilePlugins_Statistics_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_Statistics_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_Statistics_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Statistics plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginStatisticsAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_Statistics_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Statistics plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginStatistics%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Tag Range


REM # =============================================================================================
REM # Documentation: Compile the Tag Range plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_TagRange
CALL :CompileProject_CompilePlugins_TagRange_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_TagRange_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_TagRange_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Tag Range plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginTagRangeAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_TagRange_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Tag Range plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginTagRange%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM USDF


REM # =============================================================================================
REM # Documentation: Compile the USDF plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_USDF
CALL :CompileProject_CompilePlugins_USDF_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_USDF_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_USDF_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the USDF plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginUSDFAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_USDF_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the USDF plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginUSDF%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM GZDoom Virtual Mode


REM # =============================================================================================
REM # Documentation: Compile the GZDoom Virtual Mode plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_GZDoomVirtualMode
CALL :CompileProject_CompilePlugins_GZDoomVirtualMode_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_GZDoomVirtualMode_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_GZDoomVirtualMode_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the GZDoom Virtual Mode plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginGZDoomVirtualModeAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_GZDoomVirtualMode_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the GZDoom Virtual Mode plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginGZDoomVirtualMode%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM WadAuthor Virtual Mode


REM # =============================================================================================
REM # Documentation: Compile the WadAuthor plugin
REM # =============================================================================================
:CompileProject_CompilePlugins_WadAuthorVirtualMode
CALL :CompileProject_CompilePlugins_WadAuthorVirtualMode_UpdateVersion || EXIT /B %ERRORLEVEL%
CALL :CompileProject_CompilePlugins_WadAuthorVirtualMode_Compile || EXIT /B %ERRORLEVEL%
EXIT /B 0



:CompileProject_CompilePlugins_WadAuthorVirtualMode_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the WADAuthor Virtual Mode plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginWadAuthorVirtualModeAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_WadAuthorVirtualMode_Compile
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the WADAuthor Virtual Mode plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginWadAuthorVirtualMode%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Visplane Explorer


REM # =============================================================================================
REM # Documentation: Compile the Visplane Explorer plugin; this requires upgrading the source files to the latest version of Visual Studio.
REM # =============================================================================================
:CompileProject_CompilePlugins_VisplaneExplorer
CALL :CompileProject_CompilePlugins_VisplaneExplorer_UpdateVersion || EXIT /B 1
CALL :CompileProject_CompilePlugins_VisplaneExplorer_UpgradeLibrary || EXIT /B 1
REM ----
REM Can we find the file?
CALL :CompileProject_CompilePlugins_VisplaneExplorer_ResourceCheck || EXIT /B 1
REM ----
CALL :CompileProject_CompilePlugins_VisplaneExplorer_Library || EXIT /B 1
CALL :CompileProject_CompilePlugins_VisplaneExplorer_MirrorFiles || EXIT /B 1
CALL :CompileProject_CompilePlugins_VisplaneExplorer_BuildVisplaneExplorer || EXIT /B 1
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_VisplaneExplorer_ResourceCheck
CALL :CompileProject_Display_IncomingTaskSubLevel "Can Visplane Explorer's plugin support version update?"
REM Can we find the upgraded Visplane project?
CALL :CompileProject_CheckResources_CheckTargets_Process "%ProjectDirectory.PathPluginVisplaneExplorerLibraryUpgrade%" || EXIT /B 1
EXIT /B 0



:CompileProject_CompilePlugins_VisplaneExplorer_BuildVisplaneExplorer
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Visplane Explorer plugin"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginVisplaneExplorer%" %MSVSArg%
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_VisplaneExplorer_Library
CALL :CompileProject_Display_IncomingTaskSubLevel "Compiling the Visplane Explorer Dynamic Link Library"
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: MSBuild Compiler Task" /B /WAIT /%PriorityGeneral% MSBUILD "%ProjectDirectory.PathPluginVisplaneExplorerLibraryUpgrade%" %MSVSArgVPO% /property:OutDir=.\
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_VisplaneExplorer_MirrorFiles
CALL :CompileProject_Display_IncomingTaskSubLevel "Transitioning the Visplane Explorer Dynamic Link Library to the Visplane Explorer plugin directory"
SET TaskCaller_NiceProgramName=Copy
SET TaskCaller_CallLong=COPY /V /Z "%ProjectDirectory.PathPluginVisplaneExplorerLibraryDir%vpo.dll" "%ProjectDirectory.PathPluginVisplaneExplorerDir%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_VisplaneExplorer_UpgradeLibrary
CALL :CompileProject_Display_IncomingTaskSubLevel "Upgrading the Visplane Explorer project to support the latest Microsoft Visual Studio version"
SET TaskCaller_NiceProgramName=Visual Studio VCUPGRADE
SET TaskCaller_NiceProgramName=Visual Studio MSBuild
SET TaskCaller_CallLong=START "Batch Fork: Project Upgrader Task" /B /WAIT /%PriorityGeneral% VCUPGRADE "%ProjectDirectory.PathPluginVisplaneExplorerLibrary%" /OverWrite
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



:CompileProject_CompilePlugins_VisplaneExplorer_UpdateVersion
REM ----
REM Can we run this function?
IF %ProjectModify.UpdateInternalProjectVersion% EQU False EXIT /B 0
REM ----
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the Visplane Explorer plugin version"
SET TaskCaller_CallLong=START "Batch Fork: Update Project Version Task" /B /WAIT /%PriorityGeneral% "%ProjectDirectory.VersionFromSVN%" "%ProjectDirectory.PathPluginVisplaneExplorerAssembly%" -F %SVNRevisionNew%
SET TaskCaller_NiceProgramName=Update Internal Project Version ID
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                            Subversion


REM -------------------------------
REM -------------------------------
REM Update the local working copy


REM # =============================================================================================
REM # Documentation: This function will update the local Working Directory to the latest revision.
REM # =============================================================================================
:CompileProject_SVNUpdateProject
REM ----
REM Run this function?
IF %Detect_SVN% EQU False EXIT /B 0
IF %UserConfig.SVNMasterControl% EQU False EXIT /B 0
REM The user does NOT want to update the local working copy?
IF %UserConfig.SVNAllowWorkingCopyUpdate% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Updating local working copy contents"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_SVNUpdateProject_TaskUpdate || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
REM ----
EXIT /B 0



:CompileProject_SVNUpdateProject_TaskUpdate
CALL :CompileProject_Display_IncomingTaskSubLevel "Updating the SVN Local Working Copy"
REM Prepare the variables needed for the operation function.
SET TaskCaller_CallLong=SVN update "%UserConfig.DirProjectWorkingCopy%"
SET TaskCaller_NiceProgramName=Subversion Commandline
CALL :CompileProject_TaskOperation
REM Error Check
CALL :CompileProject_SVNErrorCheck %ExitCode%
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Fetch the latest revision ID


REM # =============================================================================================
REM # Documentation: This function will try to retrieve the SVN revision that the working copy is based off from.  {NOT MERGED; this is not possible}
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID
REM ----
REM Run this function?
IF %Detect_SVN% EQU False EXIT /B 0
IF %UserConfig.SVNMasterControl% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Fetching SVN revision number"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
REM Perform the tasks in multiple functions, to keep things a little sane in the code.
CALL :CompileProject_FetchSVNRevisionID_OldRevision
CALL :CompileProject_FetchSVNRevisionID_TaskNewRevision || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_FetchSVNRevisionID_CleanupResult
CALL :CompileProject_FetchSVNRevisionID_Calculate
CALL :CompileProject_FetchSVNRevisionID_CalculateRange
CALL :CompileProject_FetchSVNRevisionID_CheckUpdateCachedRevision
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: If incase the cached revision ID that is stored on the filesystem matches with the new value, then do NOT write a new [but duplicated] file.  This will prevent wear-and-tear the platter [for SSD killed cell blocks] and repetitive tasks.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_CheckUpdateCachedRevision
IF %SVNRevisionOld% EQU %SVNRevisionNew% (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "The revision difference between the newly stored revision matches with the previously stored revision, there is no new %ProjectName% updates."
    SET SVNRevisionUpdateCachedRevisionID=False
) ELSE (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "The revision difference indicates that there is %ProjectName% updates."
    SET SVNRevisionUpdateCachedRevisionID=True
)
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Setup the field and find the current revision number.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_TaskNewRevision
REM Prepare the variables needed for the error function; this is NOT tied into the operation function.
SET TaskCaller_CallLong=SVNVERSION "%UserConfig.DirProjectWorkingCopy%"
SET TaskCaller_NiceProgramName=Subversion Commandline
CALL :CompileProject_FetchSVNRevisionID_NewRevision
REM Error Check
CALL :CompileProject_SVNErrorCheck %ERRORLEVEL%
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Load the previous saved revision ID to a variable.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_OldRevision
IF EXIST "%LocalDirectory.MainRoot%\SVNVersion.txt" (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Caching previously saved revision number"
    SET /P SVNRevisionOld=< "%LocalDirectory.MainRoot%\SVNVersion.txt"
) ELSE (
    CALL :CompileProject_Display_IncomingTaskSubLevelMSG "Could not find previously saved revision number, using 'Zero' instead"
    SET SVNRevisionOld=0
)
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Fetch the working copy revision number and store it in primary memory.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_NewRevision
CALL :CompileProject_Display_IncomingTaskSubLevel "Capturing the local Working Copy's current revision"
FOR /F %%a IN ('SVNVERSION "%UserConfig.DirProjectWorkingCopy%"') DO SET SVNRevisionNew=%%a
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: It is possible that if the working copy is modified, the SVN Revision ID may contain non-integers.  For example: 1715M. This is a very special variable manipulator.  The idea is to remove any non-integer from the last line of the SVN Version number.
REM #  NOTE: Mixed Revisions is NOT supported within this check, that'll require a much more complicated procedure.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_CleanupResult
CALL :CompileProject_Display_IncomingTaskSubLevel "Thrashing non-integer characters from the newly stored revision number"
REM If the working copy is Modified, the SVN version fetched will be reported as 1715M.  Remove the Modified signature character from the variable.
SET SVNRevisionNew=%SVNRevisionNew:M=%
REM ----
REM If the working copy is Switched, the SVN version fetched will be reported as 1715S.  Remove the Switch signature character from the variable.
SET SVNRevisionNew=%SVNRevisionNew:S=%
REM ----
REM If the working copy is Partial, the SVN version fetched will be reported as 1715P.  Remove the Partial signature character from the variable.
SET SVNRevisionNew=%SVNRevisionNew:P=%
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Calculate the difference between the new revision ID against the previously saved revision ID, and store the value in memory for further computation.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_Calculate
CALL :CompileProject_Display_IncomingTaskSubLevel "Calculating the difference from the current revision number and the previous revision"
SET /A SVNRevisionRange=%SVNRevisionNew%-%SVNRevisionOld%
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Calculate the range and determine if it exceeds the HardLimit set.  If the calculated value is over the HardLimit range, then merely force the hardlimit to be the changelog limit.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_CalculateRange
CALL :CompileProject_Display_IncomingTaskSubLevel "Calculating the difference range"
REM We need to make sure that the value we have from 'CompileProject_FetchSVNRevisionID_Calculate' does not exceed the hard limit variable 'SVNRevisionRangeHardLimit'
IF %SVNRevisionRange% GTR %SVNRevisionRangeHardLimit% (
    SET SVNRevisionRange=%SVNRevisionRangeHardLimit%
) ELSE (
    IF %SVNRevisionRange% LEQ 0 SET SVNRevisionRange=%SVNRevisionRangeHardLimit%
)
GOTO :EOF



REM -------------------------------
REM -------------------------------
REM Fetch the latest revision history log


REM # =============================================================================================
REM # Documentation: Fetch the SVN revision log history.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionLogHistory
REM ----
REM Run this function?
IF %Detect_SVN% EQU False EXIT /B 0
IF %UserConfig.SVNMasterControl% EQU False EXIT /B 0
REM Did the user wanted a log history?
IF %UserConfig.SVNAllowFetchRevisionLog% EQU False (
    IF %UserConfig.SVNAllowFetchRevisionLogXML% EQU False EXIT /B 0
)
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Fetching SVN revision history log"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_FetchSVNRevisionHistory || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_FetchSVNRevisionHistoryXML || (CALL :CaughtErrorSignal& EXIT /B 1)
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Generate a changelog history in a regular text file.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionHistory
REM Did the user wanted this log type?
IF %UserConfig.SVNAllowFetchRevisionLog% EQU False EXIT /B 0
CALL :CompileProject_Display_IncomingTaskSubLevel "Fetching a standard revision change log [txt formatting]"
REM Fetch the log
SET TaskCaller_CallLong=SVN log -l %SVNRevisionRange% "%UserConfig.DirProjectWorkingCopy%" 1> "%ProjectDirectory.DirCompiledTarget%\Changelog.txt"
SET TaskCaller_NiceProgramName=Subversion
SVN log -l %SVNRevisionRange% "%UserConfig.DirProjectWorkingCopy%" 1> "%ProjectDirectory.DirCompiledTarget%\Changelog.txt"
REM Error Check
CALL :CompileProject_SVNErrorCheck %ERRORLEVEL%
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Generate a changelog history in an XML file.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionHistoryXML
IF %UserConfig.SVNAllowFetchRevisionLogXML% EQU False EXIT /B 0
CALL :CompileProject_Display_IncomingTaskSubLevel "Fetching an advanced revision change log [xml formatting]"
REM Fetch the log
SET TaskCaller_CallLong=SVN log --xml -v -l %SVNRevisionRange%  -r HEAD:1 "%UserConfig.DirProjectWorkingCopy%" 1> "%ProjectDirectory.DirCompiledTarget%\Changelog.xml"
SET TaskCaller_NiceProgramName=Subversion
SVN log --xml -v -l %SVNRevisionRange%  -r HEAD:1 "%UserConfig.DirProjectWorkingCopy%" 1> "%ProjectDirectory.DirCompiledTarget%\Changelog.xml"
REM Error Check
CALL :CompileProject_SVNErrorCheck %ERRORLEVEL%
EXIT /B %ERRORLEVEL%



REM -------------------------------
REM -------------------------------
REM Update the cached revision number in the secondary storage


REM # =============================================================================================
REM # Documentation: Store the newly SVN revision ID in the secondary storage.  This will be used next time when compiling the project.
REM # =============================================================================================
:CompileProject_SVNUpdateCachedRevision
REM ----
REM Run this function?
IF %Detect_SVN% EQU False EXIT /B 0
IF %UserConfig.SVNMasterControl% EQU False EXIT /B 0
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Updating cached revision ID"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_FetchSVNRevisionID_UpdateCachedValue
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Save the information to the specified file for future use.
REM # =============================================================================================
:CompileProject_FetchSVNRevisionID_UpdateCachedValue
CALL :CompileProject_Display_IncomingTaskSubLevel "Saving the revision [%SVNRevisionNew%] ID for future use"
ECHO %SVNRevisionNew% 1> "%LocalDirectory.MainRoot%\SVNVersion.txt"
GOTO :EOF



REM -------------------------------
REM -------------------------------
REM Error Checking


REM # =============================================================================================
REM # Parameters: [{int} ExitCode]
REM # Documentation: Error check function.  This function examines the exitcodes and determines if the software closed with an error or if was successful.
REM # =============================================================================================
:CompileProject_SVNErrorCheck
REM Did Subversion return an error?
IF %1 EQU 0 (
    EXIT /B 0
) ELSE (
    EXIT /B 1
)




REM -----------------------------------------------------------------
REM =================================================================
REM -----------------------------------------------------------------
REM                         Windows Explorer


REM # =============================================================================================
REM # Documentation: Open new windows within the desktop environment that helps the user quickly identify the new contents that has been either compiled or generated such as 7z archives and\or Inno.
REM # =============================================================================================
:CompileProject_PopupWindowsExplorer
REM ----
REM Run this function?
IF %CallExplorerCommands% EQU False (
    CALL :CompileProject_TerminateSuccessfully_ExtendedNoExplorer
    EXIT /B 0
)
REM ----
REM This variable is used to describe the drivers main purpose and present the value in the log files.
SET "DriversNiceTask=Calling Windows Explorer"
CALL :CompileProject_Display_IncomingTask "%DriversNiceTask%"
CALL :CompileProject_PopupWindowsExplorer_StandardBuild
CALL :CompileProject_PopupWindowsExplorer_Archive
CALL :CompileProject_PopupWindowsExplorer_Setup
CALL :CompileProject_DriverLogFooter "%DriversNiceTask%"
EXIT /B 0



REM # =============================================================================================
REM # Documentation: Open a new windows and select the recently compiled engine.
REM # =============================================================================================
:CompileProject_PopupWindowsExplorer_FileNotFound
ECHO !ERR: Could not locate file { %~1 } on the filesystem!  As such, the Windows Explorer will not be called to locate this file.
IF %ToggleLog% EQU True (ECHO !ERR: Could not locate file { %~1 } on the filesystem!  As such, the Windows Explorer will not be called to locate this file.)>> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Open a new windows and select the recently compiled engine.
REM # =============================================================================================
:CompileProject_PopupWindowsExplorer_StandardBuild
REM ----
REM Run this function?
IF %UserConfig.MirrorCompiledData% EQU False EXIT /B 0
IF NOT EXIST "%LocalDirectory.Builds%\%FileName%" (
    CALL :CompileProject_PopupWindowsExplorer_FileNotFound "%LocalDirectory.Builds%\%FileName%"
    EXIT /B 0
)
REM ----
SET TaskCaller_NiceProgramName=Windows Explorer
SET TaskCaller_CallLong=EXPLORER /SELECT,"%LocalDirectory.Builds%\%FileName%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Open a new windows and select the recently compiled archive data files.
REM # =============================================================================================
:CompileProject_PopupWindowsExplorer_Archive
REM ----
REM Run this function?
IF %UserConfig.7ZipMasterControl% EQU False EXIT /B 0
IF %UserConfig.7ZipGenerateArchive% EQU False EXIT /B 0
IF NOT EXIST "%LocalDirectory.Archives%\%FileName_Archive%" (
    CALL :CompileProject_PopupWindowsExplorer_FileNotFound "%LocalDirectory.Archives%\%FileName_Archive%"
    EXIT /B 0
)
REM ----
SET TaskCaller_NiceProgramName=Windows Explorer
SET TaskCaller_CallLong=EXPLORER /SELECT,"%LocalDirectory.Archives%\%FileName_Archive%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: Open a new windows environment and highlight the inno setup packaged file.
REM # =============================================================================================
:CompileProject_PopupWindowsExplorer_Setup
REM ----
REM Run this function?
IF %UserConfig.InnoMasterControl% EQU False EXIT /B 0
IF %UserConfig.InnoGenerateSetup% EQU False EXIT /B 0
IF NOT EXIST "%LocalDirectory.Setup%\%FileName_Setup%" (
    CALL :CompileProject_PopupWindowsExplorer_FileNotFound "%LocalDirectory.Setup%\%FileName_Setup%"
    EXIT /B 0
)
REM ----
SET TaskCaller_NiceProgramName=Windows Explorer
SET TaskCaller_CallLong=EXPLORER /SELECT,"%LocalDirectory.Setup%\%FileName_Setup%"
CALL :CompileProject_TaskOperation
EXIT /B %ERRORLEVEL%



REM # =============================================================================================
REM # Documentation: If the user disallows any module to use Windows Explorer, then alternatively, tell the user where the files are located within the file.
REM # =============================================================================================
:CompileProject_TerminateSuccessfully_ExtendedNoExplorer
ECHO.&ECHO.
IF %UserConfig.MirrorCompiledData% EQU True (
    ECHO Compiled Builds Directory:
    ECHO   %LocalDirectory.Builds%
    ECHO Filename:
    ECHO   %FileName%
    ECHO.
)
REM ----
IF EXIST "%LocalDirectory.Setup%\%FileName_Setup%" (
    ECHO Setup Packages Directory:
    ECHO   %LocalDirectory.Setup%
    ECHO Filename:
    ECHO   %FileName_Setup%
    ECHO.
)
REM ----
IF EXIST "%LocalDirectory.Archives%\%FileName_Archive%" (
    ECHO Archive Files Directory:
    ECHO   %LocalDirectory.Archives%
    ECHO Filename:
    ECHO   %FileName_Archive%
    ECHO.
)
GOTO :EOF




REM =====================================================================
REM Error Manager
REM ----------------------------
REM When error occurs, the methods below will output and deal with the next step without causing much hassle.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: When a resource was found missing, this function will be called.  This function will call the sub-function which outputs the error and records the error on the log.
REM # =============================================================================================
:ResourceErrorSignal
IF %ToggleLog% EQU True CALL :ResourceErrorSignal_LogError "%~1"
CALL :ResourceErrorSignal_DisplayError "%~1"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Display the error message on the user's screen from the terminal buffer.
REM # =============================================================================================
:ResourceErrorSignal_DisplayError
IF %UseBell% GEQ 1 (ECHO %SND_BELL% %SND_BELL% %SND_BELL% %SND_BELL%)
ECHO %SeparatorLong%
ECHO.&ECHO.
ECHO ^<!^>       Critical Error Has Occurred!       ^<!^>
ECHO %SeparatorLong%
ECHO.
ECHO Could not find a required file or directory, details below:
ECHO %Separator%
ECHO File or Directory that was not found:
ECHO   %~1
ECHO.
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Log the error message to the ASCII file located within the filesystem.
REM # =============================================================================================
:ResourceErrorSignal_LogError
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO.&ECHO.)>> "%STDOUT%"
(ECHO ^<!^>       Critical Error Has Occurred!       ^<!^>)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO Could not find a required file or directory, details below:)>> "%STDOUT%"
(ECHO %Separator%)>> "%STDOUT%"
(ECHO File or Directory that was not found:)>> "%STDOUT%"
(ECHO   %~1)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: When the program reaches an error during compiling process, this function will be called.  This function will accordingly output the error message on the log [depending on the ToggleLog var] and display the message on the terminal.
REM # =============================================================================================
:CaughtErrorSignal
IF %ToggleLog% EQU True CALL :CaughtErrorSignal_LogError
CALL :CaughtErrorSignal_DisplayError
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Display the error message on the user's screen from the terminal buffer.
REM # =============================================================================================
:CaughtErrorSignal_DisplayError
IF %UseBell% GEQ 1 (ECHO %SND_BELL% %SND_BELL% %SND_BELL% %SND_BELL%)
ECHO %SeparatorLong%
ECHO.&ECHO.
ECHO ^<!^>       Critical Error Has Occurred!       ^<!^>
ECHO %SeparatorLong%
ECHO.
ECHO Caught an Error Signal, details below:
ECHO %Separator%
ECHO Exit Code: %ExitCode%
ECHO Program Called: %TaskCaller_NiceProgramName%
ECHO Command Invoked: %TaskCaller_CallLong%
ECHO.
PAUSE
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Log the error message to the ASCII file located within the filesystem.
REM # =============================================================================================
:CaughtErrorSignal_LogError
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO.&ECHO.)>> "%STDOUT%"
(ECHO ^<!^>       Critical Error Has Occurred!       ^<!^>)>> "%STDOUT%"
(ECHO %SeparatorLong%)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
(ECHO Caught an Error Signal, details below:)>> "%STDOUT%"
(ECHO %Separator%)>> "%STDOUT%"
(ECHO Exit Code: %ExitCode%)>> "%STDOUT%"
(ECHO Program Called: %TaskCaller_NiceProgramName%)>> "%STDOUT%"
(ECHO Command Invoked: %TaskCaller_CallLong%)>> "%STDOUT%"
(ECHO.)>> "%STDOUT%"
GOTO :EOF




REM =====================================================================
REM Clean-up Local Directories
REM ----------------------------
REM This functionality allows the user to thrash all of the data that is stored in the local directories.
REM This helps the user to remove all of the abundant superfluous data that has accumulated within the user's secondary storage.
REM =====================================================================


REM # =============================================================================================
REM # Documentation: Cleanup menu
REM # =============================================================================================
:Cleanup_Menu
CALL :DashboardOrClassicalDisplay
CALL :Cleanup_Menu_Warning
ECHO Clean-up Local Directories Menu
ECHO %Separator%
ECHO.
ECHO Clean-up the following directories:
ECHO.
ECHO [1] Log files
ECHO [2] Compressed files
ECHO [3] Setup files
ECHO [4] Regular build files
ECHO [A] All Local Directories
ECHO [X] Exit
CALL :UserInput
CALL :Cleanup_UserInput



:Cleanup_UserInput
REM Check the parameters and determine how to execute.
IF "%STDIN%" EQU "1" GOTO :Cleanup_Choice_LogFiles
IF "%STDIN%" EQU "2" GOTO :Cleanup_Choice_CompressedFiles
IF "%STDIN%" EQU "3" GOTO :Cleanup_Choice_SetupFiles
IF "%STDIN%" EQU "4" GOTO :Cleanup_Choice_RegBuildFiles
IF /I "%STDIN%" EQU "A" GOTO :Cleanup_Choice_Everything
IF /I "%STDIN%" EQU "X" GOTO :EOF
CALL :BadInput& GOTO :Cleanup_Menu



:Cleanup_Menu_Warning
ECHO ^<?^>       WARNING       ^<?^>
ECHO %SeparatorLong%
ECHO.
ECHO Clean-up will effectively thrash the data that is located with the directories or just specific directories as requested by the user.  This feature will help the end-user to automatically flush all of the superfluous data that might have accumulated over time, which will allow the user to gain back some freespace (from their secondary storage devices).  Please note that files that are deleted by this program are not eligible for quick recovery.
ECHO.&ECHO.
GOTO :EOF



:Cleanup_Choice_LogFiles
REM Remove all of the log files
CALL :Cleanup_Thrash_LogFiles
CALL :ClearBuffer
GOTO :Cleanup_Menu



:Cleanup_Choice_CompressedFiles
REM Remove all of the compressed files
CALL :Cleanup_Thrash_CompressedFiles
CALL :ClearBuffer
GOTO :Cleanup_Menu



:Cleanup_Choice_SetupFiles
REM Remove all of the setup files
CALL :Cleanup_Thrash_SetupFiles
CALL :ClearBuffer
GOTO :Cleanup_Menu



:Cleanup_Choice_RegBuildFiles
REM Remove all of the regular build files
CALL :Cleanup_Thrash_RegBuildFiles
CALL :ClearBuffer
GOTO :Cleanup_Menu



:Cleanup_Choice_Everything
REM Remove all of the files
CALL :Cleanup_Thrash_LogFiles
CALL :Cleanup_Thrash_CompressedFiles
CALL :Cleanup_Thrash_SetupFiles
CALL :Cleanup_Thrash_RegBuildFiles
CALL :Cleanup_Thrash_CachedFiles
CALL :ClearBuffer
GOTO :Cleanup_Menu



REM # =============================================================================================
REM # Documentation: Thrash all of the log files
REM # =============================================================================================
:Cleanup_Thrash_LogFiles
ECHO Thrashing log files. . .
CALL :Cleanup_Thrash_DelCall "%LocalDirectory.Logs%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Thrash all of the archive files
REM # =============================================================================================
:Cleanup_Thrash_CompressedFiles
ECHO Thrashing compressed files. . .
CALL :Cleanup_Thrash_DelCall "%LocalDirectory.Archives%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Thrash all of the setup files
REM # =============================================================================================
:Cleanup_Thrash_SetupFiles
ECHO Thrashing setup files. . .
CALL :Cleanup_Thrash_DelCall "%LocalDirectory.Setup%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Thrash all of the regular compressed files
REM # =============================================================================================
:Cleanup_Thrash_RegBuildFiles
ECHO Thrashing Regular build files. . .
CALL :Cleanup_Thrash_DelCall "%LocalDirectory.Builds%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: Thrash all of the temporary files
REM # =============================================================================================
:Cleanup_Thrash_CachedFiles
ECHO Thrashing cached files. . .
CALL :Cleanup_Thrash_DelCall "%LocalDirectory.Temp%"
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This will display an error if incase the local directory files could not successfully expunged.
REM # =============================================================================================
:Cleanup_Thrash_Error
ECHO ^<!^>       ERROR       ^<!^>
ECHO %SeparatorLong%
ECHO.
ECHO Could not successfully thrash all of the files!
ECHO.
PAUSE
ECHO.&ECHO.
GOTO :EOF



REM # =============================================================================================
REM # Documentation: This function will allow the user to thrash a collection of files as requested.
REM # Parameters: [{string}Directory]
REM # =============================================================================================
:Cleanup_Thrash_DelCall
REM Expunge all data from the directories and subdirectories
DEL /F /Q /S "%~1\*.*" > NUL || CALL :Cleanup_Thrash_Error
REM Expunge all of the directories
FOR /D %%i IN ("%~1\*") DO RMDIR /Q /S "%%i" > NUL || CALL :Cleanup_Thrash_Error
GOTO :EOF




REM =====================================================================
REM Terminating Protocol
REM ----------------------------
REM These functions will determine how the program is going to be terminated.
REM =====================================================================



REM # =============================================================================================
REM # Documentation: When the user is ready to close the program, this function will make sure that the program reports the proper Exit Code along with any necessary statements - if needed.
REM # =============================================================================================
:Terminate_MainExit
IF %FatalExit% EQU 0 (
    SET scriptExitCode=0
    GOTO :Terminate
) ELSE (
    SET scriptExitCode=1
    GOTO :Terminate
)



REM =====================================================================
REM Termination
REM ----------------------------
REM Terminate this script
REM =====================================================================


:Terminate
ECHO.&ECHO.
ECHO Closing module...
GOTO :EOF